define(['multi-region', 'jquery', 'underscore', 'backbone', 'marionette', 'bootstrap', 'bootstrap-datepicker', 'lv-widgets', 'history-event'], function (MultiRegion) {
  var App = new Backbone.Marionette.Application();

  App.name = 'VA Moms';
  App.version = '1.0.0';
  App.showWarningInterval = 180000;    //ms
  App.sleepCheckInterval  = 10 * 2000; //ms
  App.sleepCheckThreshold = 30  * 2000; //ms

  App.AjaxLoader = new LV.AjaxLoader();
  App.Resources = new LV.Resources({path: 'va-mom-resources'});
  App.Dateman = new LV.Dateman();

  App.addRegions({
    contentRegion: '#content',
    headerRegion: 'header',
    footerRegion: 'footer',
    popupRegion: new MultiRegion({el: '#popup-region'}),
    containerRegion: '#container'
  });

  //Prevents extreme caching in IE10
  $.ajaxSetup({ cache: false });

  App.checkROA = function (app) {
    var user = app.Resources.user;
    var roaAccepted;
    if (typeof user !== 'undefined' && user.id) {
      roaAccepted = typeof user.rightOfAccessAccepted !== 'undefined' ? user.rightOfAccessAccepted : !user.rightOfAccessAccepted;
      if (!roaAccepted) {
        window.location = window.location.protocol + '//' + window.location.host + window.location.pathname + '../MobileHealthPlatformWeb/roa/index.html?url=' + window.location.pathname;
      }
    }
  };

  App.addInitializer(function () {
    _.extend(Backbone.Marionette.View.prototype, {templateHelpers: this.templateHelpers});
  });

  App.addInitializer(function () {
    $(document).on('focus', 'input', App.scrollCheck)
      .on('focus', 'a', App.scrollCheck)
      .on('focus', 'select', App.scrollCheck)
      .on('focus', 'textarea', App.scrollCheck)
      .on('focus', '.switch-btn', App.scrollCheck)
      .on('focus', 'div[role="button"]', App.scrollCheck);
  });

  App.addInitializer(function () {
    if (navigator.platform.indexOf('iPhone') !== -1 || navigator.platform.indexOf('iPad') !== -1) {
      var $body = $('body');
      $(document).on('focus', 'input', function () {
        $body.addClass('touch-keyboard');
      }).on('blur', 'input', function () {
        $body.removeClass('touch-keyboard');
      }).on('focus', 'select', function () {
        $body.addClass('touch-keyboard');
      }).on('blur', 'select', function () {
        $body.removeClass('touch-keyboard');
      }).on('focus', 'textarea', function () {
        $body.addClass('touch-keyboard');
      }).on('blur', 'textarea', function () {
        $body.removeClass('touch-keyboard');
      }).on('focus', '#save-btn', function () {
        $body.addClass('touch-keyboard');
      }).on('blur', '#save-btn', function () {
        $body.removeClass('touch-keyboard');
      });
    }
  });

  App.persistFields = [];

  App.focusPopup = function () {
    // specialized focusing
    var focus = function () {
      var $focus = $('.focus-second');
      if ($focus.length > 0) {
        $focus.focus().click();
      }
    };
    setTimeout(focus, 150);
  };

  App.focusTop = function () {
    // specialized focusing
    var focus = function () {
      var $focus = $('.focus-first');
      if ($focus.length === 0) {
        $focus = $('#header');
      }
      $focus.parent().focus();
    };
    setTimeout(focus, 150);
  };

  App.contentRegion.on('show', function () {
    App.focusTop();
  });

  App.vent.on('content:silence', function (value) {
    App.contentRegion.$el.attr('aria-hidden', value);
  });

  App.vent.on('route:startup', function () {
    Backbone.history.start();
  });

  //<----------------------------------------- Session ----------------------------------------------->

  App.login = function () {
    App.Resources.login();
  };

  App.startSessionTimer = function () {

    var timeLeft = App.getTimeToExpireInMilliSeconds();
    if (App.warningTimer) {
      clearTimeout(App.warningTimer);
    }

    if (App.endedTimer) {
      clearTimeout(App.endedTimer);
    }

    if (timeLeft > 0) {
      App.warningTimer = window.setTimeout(function () {
        App.vent.trigger('session:warning');
      }, timeLeft - App.showWarningInterval);

      App.endedTimer = window.setTimeout(function () {
        App.vent.trigger('session:ended');
      }, timeLeft);
    }

  };
  App.resetSessionTimer = function () {
    var path = App.Resources.get('user-session').href;
    $.ajax({url: path, dataType: 'json', async: false, global: false});
    App.startSessionTimer();
    return true;
  };

  App.cleanUpSession = function () {
    if (App.warningTimer) {
      clearTimeout(App.warningTimer);
    }

    if (App.endedTimer) {
      clearTimeout(App.endedTimer);
    }

    if (App.sessionSleepThread) {
      clearTimeout(App.sessionSleepThread);
    }

    App.clearSession();
  };
  App.clearSession = function () {
    window.sessionStorage.token = null;
    $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
      jqXHR.setRequestHeader('Authorization', '');
    });
    document.cookie = encodeURIComponent('JSESSIONID') + '=deleted; expires=' + new Date(0).toUTCString();
  };
  App.getJSON = function (path) {
    var response = $.ajax({url: path, dataType: 'json', async: false});
    return JSON.parse(response.responseText);
  };
  App.getTimeToExpireInMilliSeconds = function () {
    var path = App.Resources.get('last-accesstime').href;
    var that = this;
    var response = $.ajax({url: path, dataType: 'json', async: false, global: false, error: function () {
      that.vent.trigger('session:ended');
    }});

    var result;
    if (response.status === 200) {
      var json = JSON.parse(response.responseText);
      result = json.timeToExpireInSeconds;
    } else {
      result = '0';
    }

    return parseInt(result) * 1000;
  };

  //<----------------------------------------- Ajax    ----------------------------------------------->
  $(document).ajaxStart(function () {
    $('.ajax-loading').show();
  }).ajaxStop(function () {
    $('.ajax-loading').hide();
  }).ajaxError(function (event, jqxhr, settings, thrownError) {
    if (thrownError === 'timeout') {
      App.router.popups.showAjaxTimeout();
    }
  });

  //<------------------------------------------ Init ------------------------------------------------------>


  if (App.Resources.isLoggedIn()) {
    //session timeout timer
    App.startSessionTimer();
    $(document).on('ajaxSuccess', function thisfoo(e) {
      App.resetSessionTimer();
    });

    //session sleep timer
    App.lastActiveTime = new Date().getTime(); //int
    App.sessionSleepThread = window.setTimeout(function thisfoo(e) {
      var currentTime = new Date().getTime();
      var difference = currentTime - App.lastActiveTime - App.sleepCheckInterval;

      if (difference > App.sleepCheckThreshold) {
        App.vent.trigger('session:wokeUp');
      }

      App.lastActiveTime = currentTime;

      window.setTimeout(thisfoo, App.sleepCheckInterval);
    }, App.sleepCheckInterval);
  }

  App.scrollCheck = function () {
    var $header = $('header');
    var margin = [($header.find('.lower-menu').height() + $header.height()), $('footer').height()];
    var content = [($(window).scrollTop() + margin[0]), ($(window).scrollTop() + $(window).height() - margin[1])];
    var element = [$(document.activeElement).offset().top, ($(document.activeElement).offset().top + $(document.activeElement).height())];
    if (element[0] < content[0] - 1 || element[1] > content[1] - 1) {
      window.scrollTo(0, (element[0] - margin[0]));
    }
  };

  return App;

});
