require.config({
  map: {'*': {'lv-app': 'util/lv'}},
  packages: ['util/lv'],
  paths: {
    //Models
    'veteran-profile': 'model/veteranProfile',
    'journal-entry': 'model/journalEntry',
    'screening': 'model/screening',
    'diagnosis': 'model/diagnosis',
    'labs': 'model/labs',
    'medication': 'model/medication',
    'ultrasound': 'model/ultrasoundEntry',
    'vaccinations': 'model/vaccinations',
    'allergy': 'model/allergy',
    'outcome': 'model/outcome',
    'exportPregTracker': 'model/exportPregTracker',

    //Collections
    'journal-entries': 'collection/journalEntries',
    'screenings': 'collection/screenings',
    'diagnoses': 'collection/diagnoses',
    'medications': 'collection/medications',
    'ultrasound-entries': 'collection/ultrasoundEntries',
    'allergies': 'collection/allergies',
    'exportPregTrackerData': 'collection/exportPregTrackerData',

    //Views
    'dataAcceptView': 'view/popup/dataAcceptView',
    'pregnancy-outcome': 'view/pregnancyOutcome/pregnancyOutcomeLayoutView',
    'delete-view': 'view/popup/deleteView',
    'resourceLinkDirectView': 'view/popup/resourceLinkDirectView',
    'allergies-list-view': 'view/allergies/allergiesListView',
    'allergies-form-view': 'view/allergies/allergiesFormView',
    'ultrasound-list-view': 'view/ultrasounds/ultrasoundListView',
    'ultrasound-form-view': 'view/ultrasounds/ultrasoundLayoutView',
    'vaccinations-form-view': 'view/vaccinations/vaccinationsLayoutView',
    'medications-form-view': 'view/medications/medicationsFormView',
    'medications-list-view': 'view/medications/medicationsListView',
    'diagnosis-form-view': 'view/diagnosis/diagnosisFormView',
    'diagnosis-list-view': 'view/diagnosis/diagnosisListView',
    'screening-form-view': 'view/screening/screeningFormView',
    'screening-list-view': 'view/screening/screeningListView',
    'pregnancy-tracker-view': 'view/pregnancyTracker/pregnancyTrackerView',
    'session-ended-view': 'view/popup/sessionEndedView',
    'ajax-disconnect-view': 'view/popup/ajaxDisconnectView',
    'ajax-timeout-view': 'view/popup/ajaxTimeoutView',
    'session-end-warning-view': 'view/popup/sessionEndWarningView',
    'pop-up-view': 'view/popup/popUpView',
    'my-v-a-medical-record-view': 'view/MyVAMedicalRecord/myVAMedicalRecordView',
    'labs-layout-view': 'view/labs/labsLayoutView',
    'journal-list-view': 'view/journal/journalListView',
    'journal-form-view': 'view/journal/journalFormView',
    'eula-view': 'view/eulaView',
    'field-view': 'view/common/fieldView',
    'base-form-view': 'view/common/baseFormView',
    'profile-view': 'view/profile/profileView',
    'about-view': 'view/about/aboutView',
    'home-view': 'view/homeView',
    'footer-view': 'view/footerView',
    'header-view': 'view/headerView',
    'pregnancy-resources-view': 'view/pregnancyResources/pregnancyResourcesView',
    'exportPregTrackerView': 'view/exportPregnancyTracker/exportPregTrackerView',

    //Routers
    'router': 'router/router',

    //Controllers
    'controller': 'controller/controller',
    'popup-controller': 'controller/popupController',
    'session-controller' : 'controller/sessionController',

    //Utilities
    'bootstrap-datepicker': 'util/bootstrap-datepicker',
    'multi-region': 'util/multi-region',
    'template-helpers': 'util/template-helpers',
    'history-event': 'util/backbone.historyevent',

    //Libraries
    'bootstrap': '../lib/sass-bootstrap/dist/js/bootstrap',
    'jquery': '../lib/jquery/dist/jquery',
    'underscore': '../lib/underscore/underscore',
    'backbone': '../lib/backbone/backbone',
    'backbone.wreqr': '../lib/backbone.wreqr/lib/backbone.wreqr',
    'backbone.babysitter': '../lib/backbone.babysitter/lib/backbone.babysitter',
    'marionette': '../lib/marionette/lib/core/amd/backbone.marionette',
    'lv-widgets': '../lib/lv-widgets/dist/lv-widgets',
    //Plugins
    'text': '../lib/requirejs-text/text'
  },
  shim: {
    'bootstrap': {
      deps: ['jquery'],
      exports: 'jquery'
    },
    'bootstrap-datepicker': {
      'deps': ['bootstrap']
    }
  }
});

require(['app', 'router', 'template-helpers'], function (App, Router, TemplateHelpers) {
  App.templateHelpers = TemplateHelpers;
  App.addInitializer(function () {
    this.router = new Router();
    this.vent.trigger('route:startup');
  });
  App.start();
});
