define(function (require) {
  var App = require('app');
  var HeaderView = require('header-view');
  var FooterView = require('footer-view');
  var HomeView = require('home-view');
  var AboutView = require('about-view');
  var ProfileView = require('profile-view');
  var VeteranProfile = require('veteran-profile');
  var JournalEntries = require('journal-entries');
  var Screenings = require('screenings');
  var Diagnoses = require('diagnoses');
  var Medications = require('medications');
  var Ultrasounds = require('ultrasound-entries');
  var Allergies = require('allergies');
  var Labs = require('labs');
  var Vaccinations = require('vaccinations');
  var Outcome = require('outcome');
  var ExportPregTrackerData = require('exportPregTrackerData');
  var EulaView = require('eula-view');
  var PregTrackerView = require('pregnancy-tracker-view');
  var MyVAMedRecordView = require('my-v-a-medical-record-view');
  var JournalListView = require('journal-list-view');
  var JournalFormView = require('journal-form-view');
  var ScreeningListView = require('screening-list-view');
  var ScreeningFormView = require('screening-form-view');
  var DiagnosisListView = require('diagnosis-list-view');
  var DiagnosisFormView = require('diagnosis-form-view');
  var LabsLayoutView = require('labs-layout-view');
  var MedicationsListView = require('medications-list-view');
  var MedicationsFormView = require('medications-form-view');
  var UltrasoundListView = require('ultrasound-list-view');
  var UltrasoundFormView = require('ultrasound-form-view');
  var AllergiesListView = require('allergies-list-view');
  var AllergiesFormView = require('allergies-form-view');
  var VaccinationsFormView = require('vaccinations-form-view');
  var PregnancyOutcomeLayoutView = require('pregnancy-outcome');
  var PregnancyResourcesView = require('pregnancy-resources-view');
  var ExportPregTrackerView = require('exportPregTrackerView');

  return Backbone.Marionette.Controller.extend({

    initialize: function () {
      App.goBack = '';
      App.models = {};
      App.collections = {};
      App.views = {};

      App.models.profile = new VeteranProfile();
      App.models.labs = new Labs();
      App.models.vaccinations = new Vaccinations();
      App.models.outcome = new Outcome();

      App.collections.journal = new JournalEntries();
      App.collections.screenings = new Screenings();
      App.collections.diagnoses = new Diagnoses();
      App.collections.medications = new Medications();
      App.collections.ultrasounds = new Ultrasounds();
      App.collections.allergies = new Allergies();
      App.collections.exportPregTrackerData = new ExportPregTrackerData();

      App.views.header = new HeaderView();
      App.views.footer = new FooterView();
      App.views.homeView = new HomeView();
      App.views.aboutView = new AboutView();
      App.views.eulaView = new EulaView({app: App.name});
      App.views.profileView = new ProfileView();
      App.views.pregTrackerView = new PregTrackerView();
      App.views.myVAMedRecordView = new MyVAMedRecordView();
      App.views.labsView = new LabsLayoutView();
      App.views.outcomeView = new PregnancyOutcomeLayoutView();
      App.views.pregnancyResourcesView = new PregnancyResourcesView();

      App.contentRegion.on('before:show', function (view) {
        App.location = window.location.href;
        App.views.header.setHeaderContent(view.header, view.route);
        App.headerRegion.show(App.views.header);
      });
      App.footerRegion.show(App.views.footer);
    },
    showIndex: function () {
      if (App.Resources.isLoggedIn()) {
        App.checkROA(App);
      }
      App.contentRegion.show(App.views.homeView);
      $('body').removeClass('bgimage');
    },
    showEula: function () {
      App.containerRegion.show(App.views.eulaView);
      App.goBack = '';
    },
    showAbout: function () {
      App.contentRegion.show(App.views.aboutView);
      App.goBack = '';
    },
    showProfile: function () {
      App.contentRegion.show(new ProfileView());
      App.goBack = 'profile';
    },
    showPregTracker: function () {
      App.contentRegion.show(App.views.pregTrackerView);
      App.goBack = '';
    },
    showJournalList: function () {
      App.contentRegion.show(new JournalListView());
      App.goBack = 'journal';
    },
    showJournalEntry: function (entry) {
      var opts = (entry === 'new') ? {entry: entry} : {entry: parseInt(entry)};
      if (App.collections.journal.length === 0) {
        App.collections.journal.fetch({
          success: function () {
            App.contentRegion.show(new JournalFormView(opts));
          }
        });
      }
      else {
        App.contentRegion.show(new JournalFormView(opts));
      }
      App.goBack = 'journal-form';
    },
    showScreeningList: function () {
      App.contentRegion.show(new ScreeningListView());
      App.routeBack = 'profile';
      App.goBack = 'screening';
    },
    showScreeningTest: function (test) {
      var opts = (test === 'new') ? {test: test} : {test: parseInt(test)};
      if (App.collections.screenings.length === 0) {
        App.collections.screenings.fetch({
          success: function () {
            App.contentRegion.show(new ScreeningFormView(opts));
          }
        });
      }
      else {
        App.contentRegion.show(new ScreeningFormView(opts));
      }
      App.goBack = 'screening-form';
    },
    showDiagnosisList: function () {
      App.contentRegion.show(new DiagnosisListView());
      App.goBack = 'diagnosis';
    },
    showDiagnosisDetails: function (entry) {
      var opts = (entry === 'new') ? {entry: entry} : {entry: parseInt(entry)};
      if (App.collections.diagnoses.length === 0) {
        App.collections.diagnoses.fetch({
          success: function () {
            App.contentRegion.show(new DiagnosisFormView(opts));
          }
        });
      }
      else {
        App.contentRegion.show(new DiagnosisFormView(opts));
      }
      App.goBack = 'diagnosis-form';
    },
    showMyVAMedicalRecord: function () {
      App.contentRegion.show(App.views.myVAMedRecordView);
      App.goBack = '';
    },
    showLabs: function () {
      if (App.models.labs.isNew()) {
        App.models.labs.fetch({
          success: function () {
            App.views.labsView = new LabsLayoutView();
            App.contentRegion.show(App.views.labsView);
            App.views.labsView.showForms();
          }
        });
      }
      else {
        App.views.labsView = new LabsLayoutView();
        App.contentRegion.show(App.views.labsView);
        App.views.labsView.showForms();
      }
      App.goBack = 'labs';
    },
    showMedicationsList: function () {
      App.contentRegion.show(new MedicationsListView());
      App.goBack = 'medications';
    },
    showMedicationsDetails: function (entry) {
      var opts = (entry === 'new') ? {entry: entry} : {entry: parseInt(entry)};
      if (App.collections.medications.length === 0) {
        App.collections.medications.fetch({
          success: function () {
            App.contentRegion.show(new MedicationsFormView(opts));
          }
        });
      }
      else {
        App.contentRegion.show(new MedicationsFormView(opts));
      }
      App.goBack = 'medications-form';
    },
    showUltrasoundList: function () {
      App.contentRegion.show(new UltrasoundListView());
      App.goBack = 'ultrasound';
    },
    showUltrasoundDetails: function (entry) {
      var opts = (entry === 'new') ? {entry: entry} : {entry: parseInt(entry)};
      var view = null;
      if (App.collections.ultrasounds.length === 0) {
        App.collections.ultrasounds.fetch({
          success: function () {
            view = new UltrasoundFormView(opts);
            App.contentRegion.show(view);
            view.showForms();
          }
        });
      }
      else {
        view = new UltrasoundFormView(opts);
        App.contentRegion.show(view);
        view.showForms();
      }
      App.goBack = 'ultrasound-form';
    },
    showAllergyList: function () {
      App.contentRegion.show(new AllergiesListView());
      App.goBack = 'allergies';
    },
    showAllergyDetails: function (entry) {
      var opts = (entry === 'new') ? {entry: entry} : {entry: parseInt(entry)};
      if (App.collections.allergies.length === 0) {
        App.collections.allergies.fetch({
          success: function () {
            App.contentRegion.show(new AllergiesFormView(opts));
          }
        });
      }
      else {
        App.contentRegion.show(new AllergiesFormView(opts));
      }
      App.goBack = 'allergy-form';
    },
    showVaccinations: function () {
      var view = null;
      if (App.models.vaccinations.isNew()) {
        App.models.vaccinations.fetch({
          success: function () {
            view = new VaccinationsFormView();
            App.contentRegion.show(view);
            view.showForms();
          }
        });
      }
      else {
        view = new VaccinationsFormView();
        App.contentRegion.show(view);
        view.showForms();
      }
      App.goBack = 'vaccinations';
    },
    showPregnancyOutcome: function () {
      if (App.models.outcome.isNew()) {
        App.models.outcome.fetch({
          success: function () {
            App.views.outcomeView = new PregnancyOutcomeLayoutView();
            App.contentRegion.show(App.views.outcomeView);
            App.views.outcomeView.showForms();
          }
        });
      }
      else {
        App.views.outcomeView = new PregnancyOutcomeLayoutView();
        App.contentRegion.show(App.views.outcomeView);
        App.views.outcomeView.showForms();
      }
      App.goBack = 'outcome';
    },
    showPregnancyResources: function () {
      App.contentRegion.show(App.views.pregnancyResourcesView);
      App.goBack = '';
    },
    exportPregnTrackerData: function () {
      App.collections.exportPregTrackerData.fetch({
        reset: true,
        success: function (data) {
          App.views.exportPregTrackerView = new ExportPregTrackerView(data.models);
          App.contentRegion.show(App.views.exportPregTrackerView);
        }
      });

      App.goBack = 'pregnancy-tracker';

    }

  });
});
