define(function (require) {
  var App = require('app');
  var PopupHandleView = require('pop-up-view');
  var SessionEndWarningView = require('session-end-warning-view');
  var SessionEndedView   = require('session-ended-view');
  var AjaxTimeoutView = require('ajax-timeout-view');
  var AjaxDisconnectView = require('ajax-disconnect-view');
  var DeleteView = require('delete-view');
  var ResourceLinkDirectView = require('resourceLinkDirectView');
  var DataAcceptView = require('dataAcceptView');

  return Backbone.Marionette.Controller.extend({

    initialize: function () {
      this.listenTo(App.vent, 'popup:show', this.callMethod);
      this.listenTo(App.vent, 'popup:hideAll', this.hideAll);
      this.listenTo(App.vent, 'popup:destroyAll', this.destroyAll);

      this.on('showSessionWarning', this.showSessionWarning);
      this.on('showSessionEnded', this.showSessionEnded);
      this.on('showAjaxTimeout', this.showAjaxTimeout);
      this.on('showAjaxDisconnect', this.showAjaxDisconnect);
    },
    callMethod: function (target) {
      this.trigger(target);
    },
    hideAll: function () {
      $(App.popupRegion.$el).find('.modal').modal('hide');
      $('.modal-backdrop').remove();
    },
    destroyAll: function () {
      App.popupRegion.close();
    },
    showSessionWarning: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new SessionEndWarningView({
          timeLeft: App.showWarningInterval
        }),
        idNum : '7'
      }));
    },
    showSessionEnded: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new SessionEndedView(),
        idNum : '8'
      }));
    },
    showAjaxTimeout: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new AjaxTimeoutView(),
        idNum : '9'
      }));
    },
    showAjaxDisconnect: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new AjaxDisconnectView(),
        idNum : '10'
      }));
    },
    showPopup: function (opts) {
      var View = Backbone.Marionette.ItemView.extend({
        id: 'modalBox',
        tagName: 'div',
        template: _.template(opts.tpl)
      });
      App.popupRegion.show(new PopupHandleView({
        view: new View(),
        idNum : '90'
      }));
    },
    showDeletePopup: function (opts) {
      App.popupRegion.show(new PopupHandleView({
        view: new DeleteView(opts),
        idNum : '90'
      }));
    },
    showResourcesLinkDirectPopup: function (opts) {
      App.popupRegion.show(new PopupHandleView({
        view: new ResourceLinkDirectView(opts),
        idNum : '90'
      }));
    },
    showDataAcceptPopup: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new DataAcceptView(),
        idNum: '90'
      }));
    }
  });
});
