define(function (require) {
  var App = require('app');
  var Controller = require('controller');
  var PopupController = require('popup-controller');
  var SessionController = require('session-controller');

  var registerInternetDisconnectHandler = function () {
    var disconnected = false;
    $(document).ajaxSend(function (e) {
      if (!e) {
        e = window.event;
      }
      if (!navigator.onLine || ('offline' === e.type)) {
        App.vent.trigger('popup:hideAll');
        App.router.popups.showAjaxDisconnect();
        disconnected = true;
      }
    });
  };

  return Backbone.Marionette.AppRouter.extend({
    controller: new Controller(),
    popups: new PopupController(),
    session: new SessionController(),
    appRoutes: {
      '': 'showIndex',
      'about': 'showAbout',
      'profile': 'showProfile',
      'pregnancy-tracker': 'showPregTracker',
      'ob-care-journal': 'showJournalList',
      'ob-care-journal/:entry': 'showJournalEntry',
      'genetic-screening': 'showScreeningList',
      'genetic-screening/:test': 'showScreeningTest',
      'diagnosis': 'showDiagnosisList',
      'diagnosis/:entry': 'showDiagnosisDetails',
      'va-medical-record': 'showMyVAMedicalRecord',
      'labs': 'showLabs',
      'medications': 'showMedicationsList',
      'medications/:entry': 'showMedicationsDetails',
      'ultrasound': 'showUltrasoundList',
      'ultrasound/:entry': 'showUltrasoundDetails',
      'allergies': 'showAllergyList',
      'allergies/:entry': 'showAllergyDetails',
      'vaccinations': 'showVaccinations',
      'pregnancy-outcome': 'showPregnancyOutcome',
      'pregnancy-resources': 'showPregnancyResources',
      'export-data': 'exportPregnTrackerData',
    },
    initialize: function () {
      this.eulaDate = App.views.eulaView.eulaDate();
      this.eulaStore = JSON.parse(localStorage.getItem('VA Moms/eula'));
    },
    onRoute: function (name) {
      if (name === 'showIndex') {
        $('body').removeClass('bgimage');
      } else {
        $('body').addClass('bgimage');
      }
      if (localStorage.getItem('VA Moms/eula') === null) {
        this.controller.showEula();
      } else {
        this.eulaStore = JSON.parse(localStorage.getItem('VA Moms/eula'));
        if (new Date(this.eulaStore.DateAccepted) <= new Date(this.eulaDate)) {
          this.controller.showEula();
        }
        if ((name !== 'showIndex') && (name !== 'showAbout') && (name !== 'showPregnancyResources') && !App.Resources.isLoggedIn()) {
          this.navigate('');
          this.controller.showIndex();
          $('body').removeClass('bgimage');
        }

      }
      App.vent.trigger('popup:hideAll');
      registerInternetDisconnectHandler();

      if (localStorage.getItem('shareDataAccepted') === null && localStorage.getItem('VA Moms/eula') !== null) {
        App.router.popups.showDataAcceptPopup();
      }
    }
  });
});
