define(function (require) {
  var App = require('app');

  return {
    formatDate: function (date) {
      return (typeof date === 'string') ? '' : App.Dateman.convertDate(new Date(date));
    },
    dasherize: function (string) {
      return string.toLowerCase().replace(/ /g, '-');
    },
    lowerCase: function (string) {
      return string.toLowerCase();
    },
    getWeekAndDays: function (a, b) {
      var msInDay = 1000 * 60 * 60 * 24;

      var utc1 = Date.UTC(a.getUTCFullYear(), a.getUTCMonth(), a.getUTCDate());
      var utc2 = Date.UTC(b.getUTCFullYear(), b.getUTCMonth(), b.getUTCDate());

      var diff = Math.floor((utc2 - utc1) / msInDay);
      var str = '';
      var weeks = 0;
      var values = {
        ' Week': 7,
        ' Day': 1
      };
      for (var x in values) {
        var amount = Math.floor(diff / values[x]);
        if (amount >= 1) {
          if (x === ' Week') {
            str += amount + x + (amount > 1 ? 's' : '') + ' ';
            weeks = amount;
          } else {
            str += (weeks === 0 ? '' : 'and ') + amount + x + (amount > 1 ? 's' : '') + ' ';
          }
          diff -= amount * values[x];
        }
      }
      return {string: str, weeks: weeks};
    },
    getPregnancyDays: function (edd, startDate) {
      startDate = (typeof startDate === 'undefined') ? App.Dateman.getZeroHourDate() : new Date(startDate);
      if (edd) {
        var date = App.Dateman.addDays(new Date(edd), (-App.Dateman.getGestation()));
        var preg = this.getWeekAndDays(date, startDate);
        if (preg.weeks >= 44) {
          return 'You are outside of the expected range of pregnancy. <br> Please update your Pregnancy Outcome.';
        } else {
          return preg.string + ' Pregnant';
        }
      } else {
        return '0 Weeks Pregnant';
      }
    },
    getPregnancyWeek: function (edd, startDate) {
      startDate = (typeof startDate === 'undefined') ? App.Dateman.getZeroHourDate() : new Date(startDate);
      if (edd) {
        var date = App.Dateman.addDays(new Date(edd), (-App.Dateman.getGestation() - 1));
        var out = App.Dateman.dateDiffInWeeks(date, startDate);
        if (out < 0) { out = '0 Weeks'; }
        else if (out === 1) { out = (out + ' Week'); }
        else { out = (out + ' Weeks'); }
        return out;
      } else {
        return '0 Weeks';
      }
    },
    getDueDateWeek: function (edd) {
      if (edd) {
        var out = App.Dateman.dateDiffInWeeks(App.Dateman.getZeroHourDate(), new Date(edd));
        if (out < 0) { out = '0 Weeks'; }
        else if (out === 1) { out = (out + ' Week'); }
        else { out = (out + ' Weeks'); }
        return out;
      } else {
        return '0 Weeks';
      }
    },
    getPregnancyWeekNumber: function (edd) {
      if (edd) {
        var out = 40 - App.Dateman.dateDiffInWeeks(App.Dateman.getZeroHourDate(), new Date(edd));
        if (out < 0) { out = 0; }
        else { out = (out); }
        return out;
      } else {
        return 0;
      }
    },
    getModelIndex: function (collection, cid) {
      var model = App.collections[collection].get(cid);
      return App.collections[collection].indexOf(model);
    },
    getTitleIndex: function (title) {
      var array = title.split(' ');
      return (parseInt(array[array.length - 1]) > 0) ? array[array.length - 1] : '';
    },
    getAllergyReactions: function (cid) {
      var reactions = [];
      var model = App.collections.allergies.get(cid);
      var reactionKey = ['rash', 'hives', 'breathingProblems', 'vomiting', 'swelling', 'other'];
      var reactionText = [' Rash', ' Hives', ' Breathing Problems', ' Vomiting', ' Swelling', ' Other'];
      $.each(reactionKey, function (key, val) {
        if (model.get(val)) {
          reactions.push(reactionText[key]);
        }
      });
      return reactions.join(',');
    },
    getGeneticScreeningTests: function (cid) {
      var geneticTests = [];
      var model = App.collections.screenings.get(cid);
      var geneticTestKey = ['amnio', 'sickleCell', 'cystic', 'canavan', 'quad', 'counseling', 'cvs', 'thalassemia', 'taySach', 'other'];
      var geneticTestText = [' Amniocentesis', ' Sickle Cell Disease and Sickle Cell Trait', ' Cystic Fibrosis', ' Canavan Disease', ' MSAFP/AFP4/Quad screen', ' Genetic Counseling', ' Chorionic Villus Sampling (CVS)', ' Thalassemia', ' Tay Sachs', ' Other'];

      $.each(geneticTestKey, function (key, val) {
        if (model.get(val)) {
          geneticTests.push(geneticTestText[key]);
        }
      });
      return geneticTests.join(',');
    }
  };

});
