define(function (require) {
  var App = require('app');
  var form = require('text!../json/allergiesForm.json');
  var BaseFormView = require('base-form-view');
  var Allergy = require('allergy');


  return BaseFormView.extend({
    header: {
      outer: 'Add',
      inner: 'Allergy'
    },
    events: {
      'click #other': 'showOtherReactionsField',
      'keypress #other': 'showOtherReactionsField',
      'click #diagnosed': 'showDiagnosedFields',
      'keypress #diagnosed': 'showDiagnosedFields'
    },
    initialize: function (opts) {
      var options = opts || {};
      this.on('model:save', this.addAllergy);
      this.form = JSON.parse(form);
      //if the id is not given, it is null. MongoDB creates an ID in that case.
      //For example, while in the form view, click Save button once, mongo generates an ID and save the record to DB.
      //If we want to edit some field in the same record, while still in the form view, as Save button clicked again, mongo receives the model with null Id again, hence
      //creates another ID  and save it as another record.
      //therefore a randomId is created for model. randomID purpose is to create a unique model id every time there is new data being saved.
      var randomId = Math.random().toString(36).substr(2, 9);

      this.model = (typeof options.entry === 'string') ? new Allergy({id: randomId}) : App.collections.allergies.at(options.entry);
      this.listenTo(App.vent, 'item:delete', this.updateAllergy);
      this.listenTo(App.vent, 'item:canceldelete', this.cancelDelete);
      this.toDelete = false;
    },
    onShow: function () {
      var regex = new RegExp('new');
      if (regex.test(Backbone.history.getFragment())) {
        $('.form-field:last-of-type').hide();
      }
      $('.checkbox-form').first().parent().prepend('<div class="info-block divider"><p>If you are experiencing any of these reactions, please check the box</p></div>');
      this.hideFields();
    },

    hideFields: function (event) {
      var $reaction = $('#otherReactions');
      var $provider = $('#provider');
      var $facility = $('#facility');
      if (!$('#other').is(':checked')) {
        $reaction.parent().addClass('hide');
        $reaction.val('');
      } else {
        $reaction.parent().removeClass('hide');
      }
      if ($('#diagnosed').parent().find('input').is(':checked')) {
        $provider.parent().parent().removeClass('hide');
        $facility.parent().parent().removeClass('hide');
        $facility.val('');
        $provider.val('');
      } else {
        $provider.parent().parent().addClass('hide');
        $facility.parent().parent().addClass('hide');
      }
    },
    showOtherReactionsField: function () {
      var $target = $('#otherReactions');
      if ($('#other').is(':checked')) {
        $target.parent().removeClass('hide');
      }
      else {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    showDiagnosedFields: function () {
      var $provider = $('#provider');
      var $facility = $('#facility');
      if ($('#diagnosed').parent().find('input').is(':checked')) {
        $facility.parent().parent().removeClass('hide');
        $provider.parent().parent().removeClass('hide');
        $facility.val('');
        $provider.val('');
      }
      else {
        $facility.parent().parent().addClass('hide');
        $provider.parent().parent().addClass('hide');
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      if ($('input[name="deleteAllergy"]').prop('checked')) {
        App.router.popups.showDeletePopup({name: 'Allergy'});
        this.toDelete = true;
      } else {
        this.toDelete = false;
        this.updateAllergy();
      }
    },
    addAllergy: function () {
      App.collections.allergies.add(this.model);
    },
    updateAllergy: function () {
      this.fieldsValidated = true;
      this.requiredFields = [];
      this.requiredTypeOfValues = [];
      this.updateForm();
      var keys = _.keys(this.form);
      for (var i = 0; i < keys.length; i++) {
        var value = this.validateField(this.form[keys[i]]);
        if (this.fieldsValidated) {
          this.model.set(this.form[keys[i]].id, value);
        }
      }
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        if (!this.toDelete) {
          this.trigger('model:save');
        }
        var self = this;
        self.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      } else {
        this.errorMessage();
      }
    },
    cancelDelete: function () {
      var $delete = $('#deleteAllergy');
      //var label = $delete.attr('aria-label').split('.');
      //label[1] = $delete.find('.active').attr('aria-label') + ' Selected';
      //$delete.find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$delete.attr('aria-label', label.join('. '));
      $delete.focus().click();
      this.toDelete = false;
    }
  });
});
