define(function (require) {
  var App = require('app');
  var template = require('text!../html/allergiesListView.html');
  var emptyTemplate = require('text!../html/allergiesListEmptyView.html');
  var itemTemplate = require('text!../html/allergiesItem.html');


  return Backbone.Marionette.CompositeView.extend({
    id: 'allergies-list',
    className: 'list-container',
    tagName: 'div',
    itemView: Backbone.Marionette.ItemView.extend({
      className: 'allergies-link',
      template: _.template(itemTemplate),
      serializeData: function () {
        return _.extend(this.model.toJSON(), {cid: this.model.cid});
      }
    }),
    emptyView: Backbone.Marionette.ItemView.extend({
      template: _.template(emptyTemplate)
    }),
    itemViewContainer: '#list',
    template: _.template(template),
    header: {
      outer: 'Allergies'
    },
    route: 'allergies',
    initialize: function () {
      this.collection = App.collections.allergies;
      this.filterCollection = new Backbone.Collection([], { model: this.collection.model });
      this.listenTo(App.vent, 'header:add', this.newFormView);
      this.listenTo(App.vent, 'allergies:fetch', this.updateAria);
    },
    onShow: function () {
      $('#add-btn').show();
      this.collection.fetch({
        reset: true,
        success: function () {
          App.vent.trigger('allergies:fetch');
        }
      });
    },
    newFormView: function () {
      App.router.navigate('allergies/new');
      App.router.controller.showAllergyDetails('new');
    },
    updateAria: function () {
      if (this.collection.length < 1) {
        this.$el.find('.list-header').attr('aria-hidden', 'true');
      }
    }

  });
});
