define(function (require) {
  var App = require('app');
  var template = require('text!../html/baseFormView.html');
  var FieldView = require('field-view');


  return Backbone.Marionette.CompositeView.extend({
    className: 'form-container',
    tagName: 'div',
    itemView: FieldView,
    itemViewContainer: '#form',
    template: _.template(template),
    form: {},
    focused: false,
    requiredFields: [],
    requiredTypeOfValues: [],
    model: Backbone.Model,

    _defaultEvents: {
      'click .switch': 'toggleSwitch',
      'keypress .switch-container': 'toggleSwitch',
      'click #save-btn': 'saveForm'
    },
    constructor: function () {
      this.events = _.extend({}, this._defaultEvents, this.events);
      this.on('composite:collection:before:render', this.setForm);
      Backbone.Marionette.CompositeView.prototype.constructor.apply(this, arguments);
    },
    toggleSwitch: function (e) {
      if (e.type === 'keypress' && e.keyCode !== (13 || 37 || 39)) { return; }
      if (e.target.tagName === 'INPUT') { return; }
      var $label = e.type === 'keypress' ? $(e.currentTarget).find('.switch') : $(e.currentTarget);
      var oldText = $label.text();
      var newText = $label.attr('data-alt-text');
      //var label = $(e.currentTarget).attr('aria-label').split('.');
      $label.text(newText);
      $label.attr('data-alt-text', oldText);
      $label.toggleClass('active');
      $label.focus();
      var $input = $label.parent().find('input');
      if (!$input.prop('checked')) {
        $input.prop('checked', true);
      } else {
        $input.prop('checked', false);
      }
      $label.attr('aria-label', newText);
      //label[1] = $(e.currentTarget).find('.active').attr('aria-label') + ' Selected';
      //$(e.currentTarget).find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$(e.currentTarget).attr('aria-label', label.join('. '));
      this.trigger('switchToggle');
      setTimeout(function () {
        $label.focus();
      }, 2);
    },
    setForm: function () {
      for (var i = 0; i < this.form.length; i++) {
        this.form[i].value = this.model.get(this.form[i].id);
      }
      this.collection = new Backbone.Collection(this.form);
    },
    updateForm: function () {
      var form = this.form;
      var serializedArray = this.$itemViewContainer.serializeArray();
      this.$itemViewContainer.find('input:checkbox').each(function () {
        var index = _.pluck(serializedArray, 'name').indexOf(this.name);
        if (index < 0) { serializedArray.push({name: this.name, value: this.checked}); }
        else { serializedArray[index] = {name: this.name, value: this.checked}; }
      });
      var group = _.groupBy(serializedArray, function (obj) {
        var content = obj.name.split('-');
        if (content.length > 1) {
          obj.name = content[1];
          obj.index = content[2];
        }
        return content[0];
      });
      _.each(group, function (input, key) {
        var index = _.pluck(form, 'id').indexOf(key);
        if (input.length > 1) {
          var array = [];
          var placeholder = {};
          for (var i = 0; i < input.length; i++) {
            if (typeof input[i].index === 'undefined') {
              placeholder[input[i].name] = input[i].value;
            }
            else {
              if (typeof array[parseInt(input[i].index)] === 'undefined') {
                array[parseInt(input[i].index)] = {};
              }
              placeholder[input[i].name] = input[i].value;
              array[parseInt(input[i].index)] = _.extend(array[parseInt(input[i].index)], placeholder);
            }
          }
          form[index].value = (array.length > 0) ? array : placeholder;
        }
        else {
          form[index].value = input[0].value;
        }
      });
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      this.updateForm();
      this.validateForm();
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        this.trigger('model:save');
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      }
    },
    validateForm: function () {
      this.fieldsValidated = true;
      this.requiredFields = [];
      this.requiredTypeOfValues = [];
      var keys = _.keys(this.form);
      for (var i = 0; i < keys.length; i++) {
        var value = this.validateField(this.form[keys[i]]);
        if (this.fieldsValidated) {
          this.model.set(this.form[keys[i]].id, value);
        }
      }
      if (!this.fieldsValidated) { this.errorMessage(); }
    },
    validateField: function (input) {
      this.validateIsEmpty(input);
      switch (input.type) {
        case 'text':
          switch (input.validate) {
            case 'charOnly':
              this.validateIsChar(input);
              break;
            case 'numOnly':
              this.validateIsNum(input);
              break;
            case 'charNumOnly':
              this.validateIsCharAndNum(input);
              break;
            case 'phone':
              this.validateIsPhoneFormatted(input);
              break;
            case 'range':
              this.validateNumRange(input);
              break;
          }
          return input.value;
        case 'date':
          if (typeof input.value !== 'string') { input.value = ''; }
          this.validateIsDateFormatted(input);
          var out = App.Dateman.validateDate(input.value);
          return (_.isNaN(out)) ? '' : out;
        case 'switch':
          return input.value;
        case 'select':
          return input.value;
        case 'obContact':
          return input.value;
        case 'bloodPressure':
          var keys = _.keys(input.value);
          for (var i = 0; i < keys.length; i++) {
            var obj = {title: (input.title + ' ' + keys[i]), id: (input.id + '-' + keys[i]), value: input.value[keys[i]], min: input.min[i], max: input.max[i]};
            this.validateNumRange(obj);
          }
          if (parseInt(input.value.sys) < parseInt(input.value.dia)) {
            var message = 'Systolic must be greater than diastolic. Please check the value and re-enter.';
            var error = {id: '#' + (input.id + '-' + keys[0]), msg: message};
            this.requiredTypeOfValues.push(error);
            this.fieldsValidated = false;
          }
          return input.value;
        case 'textarea':
          return input.value;
        case 'checkbox':
          return input.value;
        case 'efw':
          return input.value;
        default:
          return input.value;
      }
    },
    validateIsEmpty: function (input) {
      if (input.required && input.value === '' || input.value === undefined || input.value === null) {
        var message = 'Incomplete';
        var error = {id: '#' + input.id, msg: message};
        this.requiredFields.push(error);
        this.fieldsValidated = false;
      }
    },
    validateIsChar: function (input) {
      if (input.value !== '' && !input.value.match(/^[A-Za-z ]+$/)) {
        var message = 'Invalid characters. Please re-enter using letters.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateIsNum: function (input) {
      var regex = /^\d+$/;
      if (input.value !== '' && !regex.test(input.value)) {
        var message = 'Invalid characters. Please re-enter using only numbers.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateIsCharAndNum: function (input) {
      if (input.value !== '' && !input.value.match(/^[A-Za-z0-9 ]*$/)) {
        var message = 'Invalid characters. Please re-enter using only letters and numbers.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateNumRange: function (input) {
      var regex = /^\d+$/;
      var num = parseInt(input.value);
      var message = '';
      var error = {};
      num = num || 0;
      var textPresent = false;
      if (input.value !== '' && !regex.test(input.value)) {
        message = 'Invalid characters. Please re-enter using only numbers.';
        error = {id: '#' + input.id, msg: message};
        this.fieldsValidated = false;
        textPresent = true;
      }
      if (textPresent !== true && input.value !== '' && num < input.min || num > input.max) {
        if (input.id === 'wbp' || input.id === 'currWgt') {
          message = 'Please check the number and re-enter.';
        }
        else {
          message = 'The number entered is outside expected range. Please check the number and re-enter.';
        }
        error = {id: '#' + input.id, msg: message};
        this.fieldsValidated = false;
      }

      if (!this.fieldsValidated) {
        this.requiredTypeOfValues.push(error);
      }
    },
    validateIsDateFormatted: function (input) {
      if (input.value !== '' && !input.value.match(/^((0[13578]|1[02])[\/.](0[1-9]|[12][0-9]|3[01])[\/.](18|19|20)[0-9]{2})|((0[469]|11)[\/.](0[1-9]|[12][0-9]|30)[\/.](18|19|20)[0-9]{2})|((02)[\/.](0[1-9]|1[0-9]|2[0-8])[\/.](18|19|20)[0-9]{2})|((02)[\/.]29[\/.](((18|19|20)(04|08|[2468][048]|[13579][26]))|2000))$/g)) {
        var message = 'Please re-enter date using mm/dd/yyyy format.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateIsPhoneFormatted: function (input) {
      //var regex = /^\(?([0-9]{3})\)?[-. ]?([0-9]{3})[-. ]?([0-9]{4})$/;
      var regex = /^[0-9-]{8,}$/;
      if (input.value !== '' && !regex.test(input.value)) {
        var message = 'Please re-enter phone number using a phone number format.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    clearErrors: function () {
      if ($('span').hasClass('requiredField')) {
        $('.requiredField').remove();
      }
    },
    errorMessage: function () {
      $.each(this.requiredFields, function () {
        $(this.id).prev().append(' <span class="requiredField">' + this.msg + '</span>');
      });
      $.each(this.requiredTypeOfValues, function () {
        $(this.id).prev().append(' <span class="requiredField">' + this.msg + '</span>');
      });
      if (this.requiredFields.length > 0) {
        $(this.requiredFields[0].id).focus();
        App.persistFields.push(this.requiredFields[0]);
      } else {
        $(this.requiredTypeOfValues[0].id).focus();
        App.persistFields.push(this.requiredTypeOfValues[0]);
      }
    }
  });
});
