define(function (require) {
  var App = require('app');
  var form = require('text!../json/diagnosisForm.json');
  var BaseFormView = require('base-form-view');
  var Diagnosis = require('diagnosis');


  return BaseFormView.extend({
    header: {
      outer: 'Health',
      inner: 'Conditions'
    },
    initialize: function (opts) {
      var options = opts || {};
      this.on('model:save', this.addDiagnosis);
      this.form = JSON.parse(form);
      //if the id is not given, it is null. MongoDB creates an ID in that case.
      //For example, while in the form view, click Save button once, mongo generates an ID and save the record to DB.
      //If we want to edit some field in the same record, while still in the form view, as Save button clicked again, mongo receives the model with null Id again, hence
      //creates another ID  and save it as another record.
      //therefore a randomId is created for model. randomID purpose is to create a unique model id every time there is new data being saved.
      var randomId = Math.random().toString(36).substr(2, 9);
      this.model = (typeof options.entry === 'string') ? new Diagnosis({id: randomId}) : App.collections.diagnoses.at(options.entry);
      this.listenTo(App.vent, 'item:delete', this.updateDiagnosis);
      this.listenTo(App.vent, 'item:canceldelete', this.cancelDelete);
      this.toDelete = false;
    },
    onShow: function () {
      var regex = new RegExp('new');
      if (regex.test(Backbone.history.getFragment())) {
        $('.form-field:last-of-type').hide();
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      if ($('input[name="deleteDiagnosis"]').prop('checked')) {
        App.router.popups.showDeletePopup({name: 'Condition'});
        this.toDelete = true;
      } else {
        this.toDelete = false;
        this.updateDiagnosis();
      }
    },
    addDiagnosis: function () {
      App.collections.diagnoses.add(this.model);
    },
    validateDateOfOnsetRange: function (input) {
      var date = App.Dateman.toUTC(App.Dateman.validateDate(input.value));
      var now = App.Dateman.getZeroHourDate().addDays(0).getTime();
      if (date >= now) {
        var message = 'The date can only be current or past. Please check the value and re-enter.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    updateDiagnosis: function () {
      this.fieldsValidated = true;
      this.requiredFields = [];
      this.requiredTypeOfValues = [];
      this.updateForm();
      var keys = _.keys(this.form);
      for (var i = 0; i < keys.length; i++) {
        var value = this.validateField(this.form[keys[i]]);
        if (this.form[keys[i]].id === 'onsetDate' && value !== '') {
          this.validateDateOfOnsetRange(this.form[keys[i]]);
        }
        if (this.fieldsValidated) {
          this.model.set(this.form[keys[i]].id, value);
        }
      }
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        if (!this.toDelete) {
          this.trigger('model:save');
        }
        var self = this;
        self.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      } else {
        this.errorMessage();
      }
    },
    cancelDelete: function () {
      var $delete = $('#deleteDiagnosis');
      //var label = $delete.attr('aria-label').split('.');
      //label[1] = $delete.find('.active').attr('aria-label') + ' Selected';
      //$delete.find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$delete.attr('aria-label', label.join('. '));
      $delete.focus().click();
      this.toDelete = false;
    }
  });
});



