define(function (require) {
  var App = require('app');
  var template = require('text!../html/diagnosisListView.html');
  var emptyTemplate = require('text!../html/diagnosisListEmptyView.html');
  var itemTemplate = require('text!../html/diagnosisListItem.html');


  return Backbone.Marionette.CompositeView.extend({
    id: 'diagnosis-list',
    className: 'list-container',
    tagName: 'div',
    itemView: Backbone.Marionette.ItemView.extend({
      className: 'diagnosis-link',
      template: _.template(itemTemplate),
      serializeData: function () {
        return _.extend(this.model.toJSON(), {cid: this.model.cid});
      }
    }),
    emptyView: Backbone.Marionette.ItemView.extend({
      template: _.template(emptyTemplate)
    }),
    itemViewContainer: '#list',
    template: _.template(template),
    header: {
      outer: 'Health',
      inner: 'Conditions'
    },
    route: 'diagnosis',
    initialize: function () {
      this.collection = App.collections.diagnoses;
      this.filterCollection = new Backbone.Collection([], { model: this.collection.model });
      this.listenTo(App.vent, 'header:add', this.newFormView);
      this.listenTo(App.vent, 'diagnosis:fetch', this.updateFilter);
    },
    onShow: function () {
      $('#add-btn').show();
      this.listenTo(App.vent, 'filter:toggle', this.filter);
      this.collection.fetch({
        reset: true,
        success: function () {
          App.vent.trigger('diagnosis:fetch');
        }
      });
    },
    newFormView: function () {
      App.router.navigate('diagnosis/new');
      App.router.controller.showDiagnosisDetails('new');
    },
    updateFilter: function () {
      this.filterCollection.set(this.collection.models);
      this.updateAria();
    },
    updateAria: function () {
      if (this.collection.length < 1) {
        this.$el.find('.list-header').attr('aria-hidden', 'true');
      }
    },
    filter: function () {
      var $current = $('#current');
      var $past = $('#past');
      var filteredList = [];

      if ($current.is(':checked')) {
        filteredList = this.filterCollection.where({status: false});
        this.collection.reset(filteredList);
      }
      else if ($past.is(':checked')) {
        filteredList = this.filterCollection.where({status: true});
        this.collection.reset(filteredList);

      } else {
        this.collection.reset(this.filterCollection.models);
      }
    }

  });
});
