define(function (require) {
  var App = require('app');
  var template = require('text!../html/headerView.html');

  return Backbone.Marionette.ItemView.extend({
    id: 'header',
    tagName: 'div',
    className: '',
    defaultRoute: false,
    template: _.template(template),
    defaultHeader: {
      outer: 'VA',
      inner: 'Moms'
    },
    initialize: function () {
      var self = this;
      this.currenturls = [];
//      this.nexturls = [];
      this.listenTo(Backbone.history, 'url:change', function (opts) {
        self.currenturls.push(opts.current);
//        self.nexturls.push(opts.next);
//        if (typeof self.entry === 'undefined') {
//          self.entry = opts.current;
//        }
      });
      this.listenTo(App.vent, 'route:back', this.triggerBack);
    },
    serializeData: function () {
      return {
        appName: App.name,
        header: this.header || this.defaultHeader,
        currentRoute: this.currentRoute || this.defaultRoute
      };
    },
    events: {
      'click .app-menu-toggle, #app-overlay, .app-menu a': 'handleToggleAppMenu',
      'click .user-menu-toggle, #user-overlay, .user-menu a': 'handleToggleUserMenu',
      'click #logout': 'userLogout',
      'click #back-btn': 'triggerBack',
      'click #add-btn': 'triggerAdd',
      'click .d-label': 'handleFilter',
      'keypress #switch-container-d': 'handleFilter',
      'change input[name="d-filter"]': 'handleFilter',
      'click #export-btn': 'triggerExport',

    },
    appMenuOpen: false,
    handleToggleAppMenu: function () {
      this.$el.find('.app-menu').toggleClass('on');
      this.appMenuOpen = !this.appMenuOpen;
      App.vent.trigger('content:silence', this.appMenuOpen);
      if (this.userMenuOpen) {
        this.$el.find('.user-menu').toggleClass('on');
        this.userMenuOpen = !this.userMenuOpen;
      }
      if (this.appMenuOpen) { this.$el.find('.app-menu').focus(); }
    },
    userMenuOpen: false,
    handleToggleUserMenu: function () {
      this.$el.find('.user-menu').toggleClass('on');
      this.userMenuOpen = !this.userMenuOpen;
      App.vent.trigger('content:silence', this.userMenuOpen);
      if (this.appMenuOpen) {
        this.$el.find('.app-menu').toggleClass('on');
        this.appMenuOpen = !this.appMenuOpen;
      }
      if (this.userMenuOpen) { this.$el.find('.user-menu').focus(); }
    },
    hideToggle: function (el) {
      if (App.Resources.isLoggedIn()) {
        $(el).removeClass('hidden');
      }
      else {
        $(el).addClass('hidden');
      }
    },
    userLogout: function (e) {
      e.preventDefault();
      App.Resources.logout();
    },
    onShow: function () {
      this.hideToggle('#logout');
      this.hideToggle('#app-menu-btn');
      $('#add-btn').on('click', function () {
        App.vent.trigger('form:add');
      });
    },
    setHeaderContent: function (header, route) {
      this.header = header;
      this.currentRoute = route;
    },
    triggerBack: function () {
      var self = this;
      switch (App.goBack) {
        case '':
          App.router.navigate('');
          App.router.controller.showIndex();
          break;
        case 'profile':
          var index = self.currenturls.length - 1;
          if (self.currenturls[index] === 'pregnancy-tracker') {
            App.router.navigate('pregnancy-tracker');
            App.router.controller.showPregTracker();
          } else {
            App.router.navigate('');
            App.router.controller.showIndex();
          }
          break;
        case 'journal-form':
          App.router.navigate('ob-care-journal');
          App.router.controller.showJournalList();
          break;
        case 'labs':
        case 'diagnosis':
        case 'screening':
        case 'medications':
        case 'ultrasound':
        case 'allergies':
        case 'vaccinations':
        case 'outcome':
          App.router.navigate('pregnancy-tracker');
          App.router.controller.showPregTracker();
          break;
        case 'diagnosis-form':
          App.router.navigate('diagnosis');
          App.router.controller.showDiagnosisList();
          break;
        case 'screening-form':
          App.router.navigate('genetic-screening');
          App.router.controller.showScreeningList();
          break;
        case 'medications-form':
          App.router.navigate('medications');
          App.router.controller.showMedicationsList();
          break;
        case 'ultrasound-form':
          App.router.navigate('ultrasound');
          App.router.controller.showUltrasoundList();
          break;
        case 'allergy-form':
          App.router.navigate('allergies');
          App.router.controller.showAllergyList();
          break;
        case 'pregnancy-tracker':
          App.router.navigate('pregnancy-tracker');
          App.router.controller.showPregTracker();
          break;
        default:
          App.router.navigate('');
          App.router.controller.showIndex();
      }
    },
    triggerAdd: function () {
      App.vent.trigger('header:add');
    },
    triggerExport: function () {
      App.vent.trigger('header:export');
    },
    handleFilter: function (e) {
      if (e.type === 'keypress' && e.keyCode !== 13) { return; }
      var $parent = $('.diagnosis-subhead');
      var $all = $('#show-all');
      var $current = $('#current');
      var $past = $('#past');
      if (e.type === 'keypress' && e.keyCode === 13) {
        $parent.find('.active').toggleClass('active');
        if ($all.is(':checked')) {
          $current.prop('checked', true);
          $('#2').toggleClass('active').text('Current').focus();
        }
        else if ($current.is(':checked')) {
          $past.prop('checked', true);
          $('#3').toggleClass('active').text('Past').focus();
        }
        else if ($past.is(':checked')) {
          $all.prop('checked', true);
          $('#1').toggleClass('active').text('All').focus();
        }
        App.vent.trigger('filter:toggle');
        return;
      }
      if ($all.is(':checked')) {
        $parent.find('.active').toggleClass('active');
        $('#1').toggleClass('active').text('All').focus();
      }
      else if ($current.is(':checked')) {
        $parent.find('.active').toggleClass('active');
        $('#2').toggleClass('active').text('Current').focus();
      } else {
        $parent.find('.active').toggleClass('active');
        $('#3').toggleClass('active').text('Past').focus();
      }
      App.vent.trigger('filter:toggle');
    }
  });
});
