define(function (require) {
  var App = require('app');
  var form = require('text!../json/journalForm.json');
  var BaseFormView = require('base-form-view');
  var JournalEntry = require('journal-entry');


  return BaseFormView.extend({
    header: {
      outer: 'Journal',
      inner: 'Entry'
    },
    initialize: function (opts) {
      var options = opts || {};
      this.on('model:save', this.addEntry);
      this.form = JSON.parse(form);
      //if the id is not given, it is null. MongoDB creates an ID in that case.
      //For example, while in the form view, click Save button once, mongo generates an ID and save the record to DB.
      //If we want to edit some field in the same record, while still in the form view, as Save button clicked again, mongo receives the model with null Id again, hence
      //creates another ID  and save it as another record.
      //therefore a randomId is created for model. randomID purpose is to create a unique model id every time there is new data being saved.
      var randomId = Math.random().toString(36).substr(2, 9);
      this.model = (typeof options.entry === 'string') ? new JournalEntry({id: randomId}) : App.collections.journal.at(options.entry);
      this.on('switchToggle', this.hideField);
    },
    setForm: function () {
      var bloodPressure = {sys: this.model.get('bloodPressureSys'), dia: this.model.get('bloodPressureDia')};
      for (var i = 0; i < this.form.length; i++) {
        this.form[i].value = (this.form[i].id === 'bloodPressure') ? bloodPressure : this.model.get(this.form[i].id);
      }
      this.collection = new Backbone.Collection(this.form);
    },
    onShow: function () {
      if (this.model.get('fetalHeartBeatPresent') === false) {
        $('#fetal-heart-rate').parent().toggleClass('hide');
      }
    },
    addEntry: function () {
      this.model.set('bloodPressureSys', this.model.get('bloodPressure').sys)
        .set('bloodPressureDia', this.model.get('bloodPressure').dia)
        .unset('bloodPressure');
      App.collections.journal.add(this.model);
    },
    hideField: function () {
      var $target = $('#fetal-heart-rate');
      $target.parent().toggleClass('hide');
      $target.val('');
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      this.updateForm();
      this.validateForm();
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        this.trigger('model:save');
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      }
    },
  });
});
