define(function (require) {
  var App = require('app');
  var BaseFormView = require('base-form-view');
  var template = require('text!../html/labsLayoutView.html');

  return Backbone.Marionette.Layout.extend({
    header: {
      outer: 'Labs'
    },
    template: _.template(template),

    forms: {
      initialLabsForm: require('text!../json/initialLabsForm.json'),
      positiveTestsForm: require('text!../json/positiveTestsForm.json'),
      secondTrimesterForm: require('text!../json/secondTrimesterForm.json'),
      thirdTrimesterForm: require('text!../json/thirdTrimesterForm.json'),
      labTestsForm: require('text!../json/otherTests.json')
    },

    regions: {
      initialLabsForm: '#initial-labs-form',
      positiveTestsForm: '#positive-tests-form',
      secondTrimesterForm: '#second-trimester-form',
      thirdTrimesterForm: '#third-trimester-form'
    },

    events: {
      'click #add-tests-btn': 'addLabTestsRegion',
      'click #save-btn': 'saveForm',
      'change #glucoseTolerance': 'toggleGTT'
    },

    initialize: function () {
      this.model = App.models.labs;
      var keys = _.keys(this.forms);
      for (var i = 0; i < keys.length; i++) {
        if (typeof this.forms[keys[i]] === 'string') { this.forms[keys[i]] = JSON.parse(this.forms[keys[i]]); }
      }
      this.initialLabsForm.on('show', this.hideCommentField);
    },
    toggleGTT: function (e) {
      if ($(e.currentTarget).val() === '3 Hour Glucose Test') {
        $('#gttStatus').parent().parent().parent().show();
      } else {
        $('#gttStatus').parent().parent().parent().hide();
      }
    },
    showHideWeeksBlocks: function () {
      if (this.edd < 28) {
        var $weeks28 = $('#weeks28');
        var $secondTriFormField = $('#second-trimester-form');
        $weeks28.addClass('hide');
        $secondTriFormField.addClass('hide');
      }
      if (this.edd < 36) {
        var $weeks36 = $('#weeks36');
        var $thirdTriFormField = $('#third-trimester-form');
        $weeks36.addClass('hide');
        $thirdTriFormField.addClass('hide');
      }
      if (this.model.get('labTests').length < 1) {
        this.$el.find('#otherTests').addClass('hide');
      }
      if ($('#glucoseTolerance').val() === '3 Hour Glucose Test') {
        $('#gttStatus').parent().parent().parent().show();
      } else {
        $('#gttStatus').parent().parent().parent().hide();
      }
    },
    hideCommentField: function () {
      var $target = $('#papSmearComment');
      if ($target.val().length < 1 && $('#papSmear').val() !== 'Abnormal') {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    showCommentField: function (e) {
      var $target = $('#papSmearComment');
      if (e.currentTarget.value === 'Abnormal') {
        $target.parent().removeClass('hide');
      }
      else {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    createFormOpts: function (id, model) {
      var form = [];
      for (var i = 0; i < this.forms[id].length; i++) {
        var obj = _.clone(this.forms[id][i]);
        form.push(obj);
      }
      return {
        template: _.template('<form class="labs-form" role="form"></form>'),
        itemViewContainer: '.labs-form',
        form: form,
        model: model,
        events: {
          'change #papSmear': this.showCommentField
        }
      };
    },
    showForms: function () {
      this.setLabTestsRegion();
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        var opts = '';
        if (keys[i].split('-')[0] === 'otherTestsForm') {
          var index = parseInt(keys[i].split('-')[1]);
          this.updateLabTestsForm(index);
          opts = this.createFormOpts('labTestsForm', this.createLabTestsModel(index));
        } else {
          opts = this.createFormOpts(keys[i], this.model);
        }
        var FormView = BaseFormView.extend(opts);
        this[keys[i]].show(new FormView());
        var date = App.models.profile.get('edd');
        this.edd = App.templateHelpers.getPregnancyWeekNumber(date);
      }
      this.showHideWeeksBlocks();
      var $elem = $('#rhType');
      $elem.parent().parent().parent().addClass('special');
      $elem.parent().removeClass('col-xs-9').addClass('col-xs-12');
      $('#antibodyScreen').parent().removeClass('col-xs-9').addClass('col-xs-10');
    },
    createLabTestsModel: function (index) {
      var attributes = {};
      for (var i = 0; i < this.forms.labTestsForm.length; i++) {
        if (typeof index === 'undefined') {
          attributes[this.forms.labTestsForm[i].id] = '';
        }
        else {
          attributes[this.forms.labTestsForm[i].id] =
            (typeof this.model.get('labTests')[index][this.forms.labTestsForm[i].id.split('-')[1]] === 'undefined') ? '' :
              this.model.get('labTests')[index][this.forms.labTestsForm[i].id.split('-')[1]];
        }
      }
      return new Backbone.Model(attributes);
    },
    updateLabTestsModel: function (view) {
      var model = view.model.toJSON();
      var attributes = {};
      var keys = _.keys(model);
      for (var i = 0; i < keys.length; i++) {
        attributes[keys[i].split('-')[1]] = model[keys[i]];
      }
      view.model = new Backbone.Model(attributes);
    },
    updateLabTestsForm: function (index) {
      for (var i = 0; i < this.forms.labTestsForm.length; i++) {
        var id = this.forms.labTestsForm[i].id.split('-');
        id[2] = index;
        this.forms.labTestsForm[i].id  = id.join('-');
        var title = this.forms.labTestsForm[i].title.split(' ');
        title[3] = (index + 1);
        this.forms.labTestsForm[i].title = title.join(' ');
      }
    },
    validateLabTestsForm: function (view) {
      var labTests = view.$itemViewContainer.serializeArray();
      for (var i = 0; i < labTests.length; i++) {
        view.form[i].value = labTests[i].value;
      }
      view.validateForm();
    },
    setLabTestsRegion: function () {
      for (var i = 0; i < this.model.get('labTests').length; i++) {
        this.regions[('otherTestsForm-' + i)] = ('#other-tests-form-' + i);
        this.appendFormsHtml(('other-tests-form-' + i));
        this.addRegion(('otherTestsForm-' + i), ('#other-tests-form-' + i));
      }
    },
    addLabTestsRegion: function (e) {
      e.preventDefault();
      if (this.$el.find('#otherTests').hasClass('hide')) {
        this.$el.find('#otherTests').removeClass('hide');
      }
      var index = this.$el.find('.lab-tests').length;
      var id = 'other-tests-form-' + index;
      var regionName = 'otherTestsForm-' + index;
      this.regions[regionName] = '#' + id;
      this.appendFormsHtml(id);
      this.addRegion(regionName, '#' + id);
      this.updateLabTestsForm(index);
      var opts = this.createFormOpts('labTestsForm', this.createLabTestsModel());
      var FormView = BaseFormView.extend(opts);
      this[regionName].show(new FormView());
      $('#' + id).find('input').first().focus();
    },
    appendFormsHtml: function (id) {
      var div = document.createElement('div');
      div.id = id;
      div.classList.add('lab-tests');
      this.$el.find('#forms-container').append(div);
    },
    saveForm: function (e) {
      e.preventDefault();
      this.initialLabsForm.currentView.clearErrors();
      var formName = '';
      var labTests = [];
      this.fieldsValidated = true;
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        formName = keys[i].split('-');
        if (formName[0] === 'otherTestsForm') {
          this.validateLabTestsForm(this[keys[i]].currentView);
        } else {
          this[keys[i]].currentView.updateForm();
          this[keys[i]].currentView.validateForm();
        }
        if (!this[keys[i]].currentView.fieldsValidated) {
          this.fieldsValidated = false;
        }
        if (formName[0] === 'otherTestsForm' && this.fieldsValidated) {
          this.updateLabTestsModel(this[keys[i]].currentView);
          labTests.push(this[keys[i]].currentView.model.toJSON());
        }
      }

      if (this.fieldsValidated) {
        this.model.set('labTests', labTests);
        this.model.set('veteranId', App.Resources.user.id);
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      }
    }
  });
});
