define(function (require) {
  var App = require('app');
  var form = require('text!../json/medicationsForm.json');
  var BaseFormView = require('base-form-view');
  var Medication = require('medication');


  return BaseFormView.extend({
    header: {
      outer: 'Medication',
      inner: 'Details'
    },
    events: {
      'click #unwantedSideEffects': 'showCommentField',
      'keypress #unwantedSideEffects': 'showCommentField'
    },
    initialize: function (opts) {
      var options = opts || {};
      this.on('model:save', this.addMedication);
      this.form = JSON.parse(form);
      //if the id is not given, it is null. MongoDB creates an ID in that case.
      //For example, while in the form view, click Save button once, mongo generates an ID and save the record to DB.
      //If we want to edit some field in the same record, while still in the form view, as Save button clicked again, mongo receives the model with null Id again, hence
      //creates another ID  and save it as another record.
      //therefore a randomId is created for model. randomID purpose is to create a unique model id every time there is new data being saved.
      var randomId = Math.random().toString(36).substr(2, 9);
      this.model = (typeof options.entry === 'string') ? new Medication({id: randomId}) : App.collections.medications.at(options.entry);
      this.listenTo(App.vent, 'item:delete', this.updateMedication);
      this.listenTo(App.vent, 'item:canceldelete', this.cancelDelete);
      this.toDelete = false;
    },
    onShow: function () {
      var regex = new RegExp('new');
      if (regex.test(Backbone.history.getFragment())) {
        $('.form-field:last-of-type').hide();
      }
      this.hideCommentField();
    },
    hideCommentField: function () {
      var $target = $('#sideEffectsComment');
      if (!$('#unwantedSideEffects').parent().find('input').prop('checked')) {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    showCommentField: function () {
      var $target = $('#sideEffectsComment');
      if ($('#unwantedSideEffects').parent().find('input').prop('checked')) {
        $target.parent().removeClass('hide');
      }
      else {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      if ($('input[name="deleteMedication"]').prop('checked')) {
        App.router.popups.showDeletePopup({name: 'Medication'});
        this.toDelete = true;
      } else {
        this.toDelete = false;
        this.updateMedication();
      }
    },
    addMedication: function () {
      App.collections.medications.add(this.model);
    },
    validateLastRefillDate: function (input) {
      var date = App.Dateman.toUTC(App.Dateman.validateDate(input.value));
      var now = App.Dateman.getZeroHourDate().addDays(0).getTime();
      if (date > now) {
        var message = 'The date can only be current or past. Please check the value and re-enter.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateStopDate: function (stop, start) {
      var stopDate = App.Dateman.toUTC(App.Dateman.validateDate(stop.value));
      var startDate = App.Dateman.toUTC(App.Dateman.validateDate(start.value));
      if (stopDate <= startDate) {
        var message = 'The stop date of a medication cannot precede its start date. Please check the value and re-enter.';
        var error = {id: '#' + stop.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    updateMedication: function () {
      this.fieldsValidated = true;
      this.requiredFields = [];
      this.requiredTypeOfValues = [];
      this.updateForm();
      var keys = _.keys(this.form);
      for (var i = 0; i < keys.length; i++) {
        var value = this.validateField(this.form[keys[i]]);
        if (this.form[keys[i]].id === 'lastRefillDate' && value !== '') {
          this.validateLastRefillDate(this.form[keys[i]]);
        }
        if (this.form[keys[i]].id === 'stopDate' && value !== '') {
          var startDate;
          for (var j = 0; j < keys.length; j++) {
            if (this.form[keys[j]].id === 'startDate') {
              startDate = this.form[keys[j]];
            }
          }
          this.validateStopDate(this.form[keys[i]], startDate);
        }
        if (this.fieldsValidated) {
          this.model.set(this.form[keys[i]].id, value);
        }
      }
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        if (!this.toDelete) {
          this.trigger('model:save');
        }
        var self = this;
        self.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      } else {
        this.errorMessage();
      }
    },
    cancelDelete: function () {
      var $delete = $('#deleteMedication');
      //var label = $delete.attr('aria-label').split('.');
      //label[1] = $delete.find('.active').attr('aria-label') + ' Selected';
      //$delete.find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$delete.attr('aria-label', label.join('. '));
      $delete.focus().click();
      this.toDelete = false;
    }
  });
});
