define(function (require) {
  var App = require('app');
  var BaseFormView = require('base-form-view');
  var template = require('text!../html/pregnancyOutcomeLayoutView.html');
  var form = require('text!../json/pregnancyOutcomeForm.json');

  return Backbone.Marionette.Layout.extend({
    header: {
      outer: 'Pregnancy',
      inner: 'Outcome'
    },
    template: _.template(template),

    forms: {
      infantForm: form
    },

    regions: {
    },

    events: {
      'click #add-infant-btn': 'addInfantRegion',
      'click #save-btn': 'saveForm',
      'change select': 'toggleFieldsOnChange'
    },

    initialize: function () {
      this.model = App.models.outcome;
      var keys = _.keys(this.forms);
      for (var i = 0; i < keys.length; i++) {
        if (typeof this.forms[keys[i]] === 'string') { this.forms[keys[i]] = JSON.parse(this.forms[keys[i]]); }
      }
    },
    onShow: function () {

    },
    toggleFieldsOnChange: function (e) {
      var index = e.currentTarget.id.split('-')[2];
      var sex = '#infants-sex-' + index;
      var weight = '#infants-weight-' + index;
      var length = '#infants-length-' + index;
      var first = '#infants-firstName-' + index;
      var last = '#infants-lastName-' + index;
      var other = '#infants-otherOutcome-' + index;
      var liveOptions = [sex, weight, length, first, last];
      if ($(e.currentTarget).val() === 'Vaginal Delivery' || $(e.currentTarget).val() === 'C-Section') {
        $(other).parent().hide();
        for (var i = 0; i < liveOptions.length; i++) {
          $(liveOptions[i]).parent().parent().show();
        }
      }
      else if ($(e.currentTarget).val() === 'Other') {
        $(other).parent().show();
        for (var j = 0; j < liveOptions.length; j++) {
          $(liveOptions[j]).parent().parent().hide();
        }
      } else {
        $(other).parent().hide();
        for (var k = 0; k < liveOptions.length; k++) {
          $(liveOptions[k]).parent().parent().hide();
        }
      }
    },
    toggleFieldsOnLoad: function () {
      for (var i = 0; i < $('select').length; i++) {
        var select = '#infants-outcome-' + i;
        var other = '#infants-otherOutcome-' + i;
        var one = '#infants-sex-' + i;
        var two = '#infants-weight-' + i;
        var three = '#infants-length-' + i;
        var four = '#infants-firstName-' + i;
        var five = '#infants-lastName-' + i;
        var toToggle = [one, two, three, four, five];
        if ($(select).val() === 'Vaginal Delivery' || $(select).val() === 'C-Section') {
          for (var j = 0; j < toToggle.length; j++) {
            $(toToggle[j]).parent().parent().show();
          }
        } else {
          for (var k = 0; k < toToggle.length; k++) {
            $(toToggle[k]).parent().parent().hide();
          }
        }
        if ($(select).val() === 'Other') {
          $(other).parent().show();
        } else {
          $(other).parent().hide();
        }
      }
    },
    addInfantRegion: function (e) {
      e.preventDefault();
      var index = this.$el.find('.infants').length;
      var id = 'infant-form-' + index;
      var regionName = 'infantForm-' + index;
      this.regions[regionName] = '#' + id;
      this.appendFormsHtml(id);
      this.addRegion(regionName, '#' + id);
      this.updateInfantForm(index);
      var opts = this.createFormOpts('infantForm', this.createNewInfantModel());
      var FormView = BaseFormView.extend(opts);
      this[regionName].show(new FormView());
      $('.title').last().focus();
    },
    appendFormsHtml: function (id) {
      var index = parseInt(id.split('-')[2]);
      var div = document.createElement('div');
      div.id = 'infant-' + index;
      div.classList.add('header-block');
      div.classList.add('title');
      $(div).attr('tabindex', '0');
      $(div).attr('aria-live', 'assertive');
      $(div).attr('aria-label', ('outcome ' + (index + 1)));
      div.innerHTML = '<p>Outcome ' + (index + 1) + '</p>';
      this.$el.find('#forms-container').append(div);
      div = document.createElement('div');
      div.id = id;
      div.classList.add('infants');
      this.$el.find('#forms-container').append(div);
    },
    updateInfantForm: function (index) {
      for (var i = 0; i < this.forms.infantForm.length; i++) {
        var id = this.forms.infantForm[i].id.split('-');
        id[2] = index;
        this.forms.infantForm[i].id  = id.join('-');
      }
    },
    createFormOpts: function (id, model) {
      var form = [];
      for (var i = 0; i < this.forms[id].length; i++) {
        var obj = _.clone(this.forms[id][i]);
        form.push(obj);
      }
      return {
        template: _.template('<form class="outcome-form" role="form"></form>'),
        itemViewContainer: '.outcome-form',
        form: form,
        model: model
      };
    },
    createNewInfantModel: function (index) {
      var attributes = {};
      for (var i = 0; i < this.forms.infantForm.length; i++) {
        if (typeof index === 'undefined') {
          attributes[this.forms.infantForm[i].id] = '';
        }
        else {
          attributes[this.forms.infantForm[i].id] =
            (typeof this.model.get('outcomeList')[index][this.forms.infantForm[i].id.split('-')[1]] === 'undefined') ? '' :
              this.model.get('outcomeList')[index][this.forms.infantForm[i].id.split('-')[1]];
        }
      }
      return new Backbone.Model(attributes);
    },
    showForms: function () {
      this.setInfantRegion();
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        var opts = '';
        if (keys[i].split('-')[0] === 'infantForm') {
          var index = parseInt(keys[i].split('-')[1]);
          this.updateInfantForm(index);
          opts = this.createFormOpts('infantForm', this.createNewInfantModel(index));
        } else {
          opts = this.createFormOpts(keys[i], this.model);
        }
        var FormView = BaseFormView.extend(opts);
        this[keys[i]].show(new FormView());
      }
      this.toggleFieldsOnLoad();
    },
    setInfantRegion: function () {
      for (var i = 0; i < this.model.get('outcomeList').length; i++) {
        this.regions[('infantForm-' + i)] = ('#infant-form-' + i);
        this.appendFormsHtml(('infant-form-' + i));
        this.addRegion(('infantForm-' + i), ('#infant-form-' + i));
      }
    },
    updateInfantModel: function (view) {
      var model = view.model.toJSON();
      var attributes = {};
      for (var i = 0; i < view.form.length; i++) {
        attributes[view.form[i].id.split('-')[1]] = model[view.form[i].id];
      }
      if (attributes.outcome !== 'Vaginal Delivery' && attributes.outcome !== 'C-Section') {
        attributes.firstName = '';
        attributes.lastName = '';
        attributes.length = '';
        attributes.weight = '';
        if (attributes.outcome !== 'Other') {
          attributes.otherOutcome = '';
        }
      }
      else {
        attributes.otherOutcome = '';
      }
      view.model = new Backbone.Model(attributes);
    },
    validateInfantForm: function (view) {
      var infants = view.$itemViewContainer.serializeArray();
      var endDate;
      var hasSexValue = false;
      for (var x = 0; x < infants.length; x++) {
        if (infants[x].name.split('-')[1] === 'sex') {
          hasSexValue = true;
        }
      }
      for (var i = 0; i < infants.length; i++) {
        if (view.form[i].id.split('-')[1] === 'sex') {
          view.form[i].value = hasSexValue;
        } else {
          view.form[i].value = infants[i].value;
        }
      }

      view.validateForm();
      for (var j = 0; j < infants.length; j++) {
        if (view.form[j].id.split('-')[1] === 'endDate') {
          endDate = view.form[j];
        }
      }
      this.validateEndDate(endDate);
    },
    validateEndDate: function (endDate) {
      if (endDate.value !== '') {
        var date = App.Dateman.toUTC(App.Dateman.validateDate(endDate.value));
        var now = App.Dateman.getZeroHourDate().addDays(0).getTime();
        if (date > now) {
          var message = 'The date can only be current or past. Please check the value and re-enter.';
          var error = {id: endDate.id, msg: message};
          $('#' + error.id).prev().append(' <span class="requiredField">' + error.msg + '</span>');
          App.persistFields.push({id: '#' + error.id});
          this.fieldsValidated = false;
        }
      } else {
        this.fieldsValidated = false;
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      App.persistFields = [];
      var formName = '';
      var infants = [];
      this.fieldsValidated = true;
      var keys = _.keys(this.regions);
      for (var j = 0; j < keys.length; j++) {
        this[keys[j]].currentView.clearErrors();
      }
      for (var i = 0; i < keys.length; i++) {
        formName = keys[i].split('-');
        this.validateInfantForm(this[keys[i]].currentView);
        if (!this[keys[i]].currentView.fieldsValidated) {
          this.fieldsValidated = false;
        }
        if (this.fieldsValidated) {
          this.updateInfantModel(this[keys[i]].currentView);
          infants.push(this[keys[i]].currentView.model.toJSON());
        }
      }
      if (this.fieldsValidated) {
        this.model.set('outcomeList', infants);
        this.model.set('veteranId', App.Resources.user.id);
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');

          }
        });
      }
      if (!this.fieldsValidated) {
        console.log(App.persistFields);
        $(App.persistFields[0].id).focus();
      }
    }
  });
});
