define(function (require) {
  var App = require('app');
  var form = require('text!../json/profileForm.json');
  var BaseFormView = require('base-form-view');
  var EddPopupTemplate = require('text!../html/popup/eddToolTipView.html');
  var MccPopupTemplate = require('text!../html/popup/mccToolTipView.html');


  return BaseFormView.extend({
    header: {
      outer: 'My',
      inner: 'Profile'
    },

    events: {
      'change .dateform': 'checkDateField',
      'click #obContacts-add-btn': 'editObContact',
      'click .remove-btn': 'editObContact',
      'click #edd-info-btn, #mcc-info-btn': 'handleTooltip',
      'keypress #edd-info-btn, #mcc-info-btn': 'handleTooltip',
      'keypress #obContacts-add-btn': 'editObContact',
      'keypress .remove-btn': 'editObContact'
    },
    initialize: function () {
      this.form = JSON.parse(form);
      this.model = App.models.profile;
    },
    onShow: function () {
      $('#lmp-icon').datepicker('setEndDate', App.Dateman.getZeroHourDate());
      $('#edd-icon').datepicker('setStartDate', App.Dateman.getZeroHourDate());
      $('.form-container').prepend('<div class="info-block"><p>Any data entered in this app will be shared with, but not necessarily reviewed by, your Maternity Care Coordinator (MCC). Please coordinate with your MCC.</p></div>');
      var eddLabel, mccLabel;
      eddLabel = $('label[for="edd"]');
      mccLabel = $('label[for="mcc"]');
      $('<span class="eddmsg"><i tabindex="0" id="edd-info-btn" class="icon-56" aria-label="Estimated Due Date. Tap for further description."></i></span>').insertAfter($(eddLabel));
      $('<span class="mccmsg"><i tabindex="0" id="mcc-info-btn" class="icon-56" aria-label="Maternity Care Coordinator (MCC). Tap for further description."></i></span>').insertAfter($(mccLabel));
      this.updateObContactControl(false);
    },
    handleTooltip: function (e) {
      if (e.type === 'keypress' && e.keyCode !== 13) { return; }
      var template;
      if (e.currentTarget.id === 'edd-info-btn') {
        template = EddPopupTemplate;
      } else if (e.currentTarget.id === 'mcc-info-btn') {
        template = MccPopupTemplate;
      }
      App.router.popups.showPopup({tpl: template});
    },
    checkDateField: function (e) {
      if (this.model.get('lmp') === '') {
        var $target = $(e.currentTarget);
        if ($target.attr('id') === 'lmp') {
          var datValue = App.Dateman.validateDate($target.val());
          if (datValue) {
            var datAdd = App.Dateman.addDays(datValue, App.Dateman.getGestation());
            var datOut = App.Dateman.convertDate(datAdd);
            var $eddBox = $('#edd');
            $eddBox.val(datOut).change();
          }
        }
      }
    },
    editObContact: function (e) {
      if (e.type === 'keypress' && e.keyCode !== 13 || e.currentTarget.className.indexOf('active') < 0) { return; }
      this.updateForm();
      var obContacts = _.find(this.form, function (obj) { return obj.id === 'obContacts'; }).value;
      if (e.currentTarget.id.split('-')[1] === 'add') {
        obContacts.push({contactInfo: '', contactType: ''});
      }
      else {
        var index = parseInt(e.currentTarget.id.split('-')[3]);
        obContacts.splice(index, 1);
      }
      this.collection.get('obContacts').set('value', obContacts);
      this.children._views[this.children._indexByModel[this.collection.get('obContacts').cid]].render();
      this.updateObContactControl(true);
    },
    updateObContactControl: function (focus) {
      var count = _.find(this.form, function (obj) { return obj.id === 'obContacts'; }).value.length;
      if (count === 1 || count < 3) {
        $('#obContacts-add-btn').addClass('active').attr('tabindex', '0');
      } else {
        var $addBtn = $('#obContacts-add-btn');
        var $label1 = $addBtn.attr('aria-label');
        $addBtn.attr('aria-label', 'Disabled. ' + $label1);
      }
      if (count !== 1) {
        $('.remove-btn').addClass('active').attr('tabindex', '0');
      } else {
        var $removeBtn = $('.remove-btn');
        var $label2 = $removeBtn.attr('aria-label');
        $removeBtn.attr('aria-label', 'Disabled. ' + $label2);
      }
      if (focus) { $('#obContacts-contactInfo-' + (count - 1)).focus(); }
    },
    validateLmpRange: function (input) {
      var date = App.Dateman.toUTC(App.Dateman.validateDate(input.value));
      var now = App.Dateman.getZeroHourDate().addDays(0).getTime();
      if (date > now) {
        var message = 'The date can only be current or past. Please check the value and re-enter.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    validateEddRange: function (input) {
      var date = (typeof App.models.profile.get('lmp') === 'string') ? App.Dateman.getZeroHourDate() : App.Dateman.validateDate(App.models.profile.get('lmp'));
      var edd = App.Dateman.validateDate(input.value);
      var diff = App.Dateman.dateDiffInWeeks(date, edd);
      if ((diff > input.maxWeeks) || (edd < App.Dateman.getZeroHourDate().getTime())) {
        var message = 'The date entered is outside expected range. Please check the value and re-enter.';
        var error = {id: '#' + input.id, msg: message};
        this.requiredTypeOfValues.push(error);
        this.fieldsValidated = false;
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      this.clearErrors();
      this.fieldsValidated = true;
      this.requiredFields = [];
      this.requiredTypeOfValues = [];
      this.updateForm();
      var keys = _.keys(this.form);
      for (var i = 0; i < keys.length; i++) {
        var value = this.validateField(this.form[keys[i]]);
        if (this.form[keys[i]].id === 'lmp') {
          if (this.form[keys[i]].value !== '' && this.fieldsValidated) { this.validateLmpRange(this.form[keys[i]]); }
        }
        if (this.form[keys[i]].id === 'edd') {
          if (this.form[keys[i]].value !== '' && this.fieldsValidated) { this.validateEddRange(this.form[keys[i]]); }
        }
        if (this.fieldsValidated) {
          this.model.set(this.form[keys[i]].id, value);
        }
      }
      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        this.trigger('model:save');
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      } else {
        this.errorMessage();
      }
    }
  });
});
