define(function (require) {
  var App = require('app');
  var form = require('text!../json/screeningForm.json');
  var BaseFormView = require('base-form-view');
  var Screening = require('screening');


  return BaseFormView.extend({
    header: {
      outer: 'Genetic Screening',
      inner: 'Details'
    },
    events: {
      'click #other': 'showOtherTestNameField',
      'keypress #other': 'showOtherTestNameField',
      'click #save-btn': 'saveForm'
    },
    initialize: function (opts) {
      var options = opts || {};
      this.on('model:save', this.addTest);
      this.form = JSON.parse(form);
      //if the id is not given, it is null. MongoDB creates an ID in that case.
      //For example, while in the form view, click Save button once, mongo generates an ID and save the record to DB.
      //If we want to edit some field in the same record, while still in the form view, as Save button clicked again, mongo receives the model with null Id again, hence
      //creates another ID  and save it as another record.
      //therefore a randomId is created for model. randomID purpose is to create a unique model id every time there is new data being saved.
      var randomId = Math.random().toString(36).substr(2, 9);
      this.model = (typeof options.test === 'string') ? new Screening({id : randomId}) : App.collections.screenings.at(options.test);
      this.listenTo(App.vent, 'item:delete', this.updateScreening);
      this.listenTo(App.vent, 'item:canceldelete', this.cancelDelete);
      this.toDelete = false;
    },
    onShow: function () {
      this.hideFields();
    },
    addTest: function () {
      App.collections.screenings.add(this.model);
    },
    hideFields: function () {
      var $othertest = $('#otherTestName');
      if (!$('#other').is(':checked')) {
        $othertest.parent().addClass('hide');
        $othertest.val('');
      } else {
        $othertest.parent().removeClass('hide');
      }
      //hide or show delete button
      var regex = new RegExp('new');
      if (regex.test(Backbone.history.getFragment())) {
        $('.form-field:last-of-type').hide();
      }
    },
    showOtherTestNameField: function () {
      var $target = $('#otherTestName');
      if ($('#other').is(':checked')) {
        $target.parent().removeClass('hide');
      }
      else {
        $target.parent().addClass('hide');
        $target.val('');
      }
    },
    saveForm: function (e) {
      var that = this;
      e.preventDefault();
      this.clearErrors();
      if ($('input[name="deleteScreening"]').prop('checked')) {
        App.router.popups.showDeletePopup({name: 'Genetic Screening'});
        this.toDelete = true;
      } else {
        this.toDelete = false;
        this.updateScreening();
      }
    },
    updateScreening: function () {
      var self = this;
      this.updateForm();
      this.validateForm();
      if (self.fieldsValidated) {
        self.model.set('veteranId', App.Resources.user.id);
        if (self.model.isNew() && !self.toDelete) {
          self.trigger('model:save');
        }
        self.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });

      }
    },
    cancelDelete: function () {
      var $delete = $('#deleteScreening');
      //var label = $delete.attr('aria-label').split('.');
      //label[1] = $delete.find('.active').attr('aria-label') + ' Selected';
      //$delete.find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$delete.attr('aria-label', label.join('. '));
      $delete.focus().click();
      this.toDelete = false;
    }
  });
});
