define(function (require) {
  var App = require('app');
  var BaseFormView = require('base-form-view');
  var template = require('text!../html/ultrasoundLayoutView.html');
  var Ultrasound = require('ultrasound');
  var form = require('text!../json/ultrasoundDetails.json');
  var PopupTemplate = require('text!../html/popup/ultrasoundsProfileNoticeView.html');

  return Backbone.Marionette.Layout.extend({
    header: {
      outer: 'Ultrasound',
      inner: 'Details'
    },
    template: _.template(template),

    forms: {
      ultrasoundDateForm: require('text!../json/dateOfUltrasoundForm.json'),
      addFetusForm: require('text!../json/ultrasoundDetails.json')
    },

    regions: {
      ultrasoundDateForm: '#ultrasound-date'
    },

    events: {
      'click #add-fetus-btn': 'addFetusRegion',
      'click #save-btn': 'saveForm'
    },

    initialize: function (opts) {
      var options = opts || {};
      this.model = (typeof options.entry === 'string') ? new Ultrasound() : App.collections.ultrasounds.at(options.entry);
      var keys = _.keys(this.forms);
      for (var i = 0; i < keys.length; i++) {
        if (typeof this.forms[keys[i]] === 'string') { this.forms[keys[i]] = JSON.parse(this.forms[keys[i]]); }
      }
      this.listenTo(App.vent, 'item:delete', this.updateUltrasound);
      this.listenTo(App.vent, 'item:canceldelete', this.cancelDelete);
      this.toDelete = false;
    },
    onShow: function () {
    },
    addFetusRegion: function (e) {
      e.preventDefault();
      var index = this.$el.find('.fetuses').length;
      var id = 'add-fetus-form-' + index;
      var regionName = 'addFetusForm-' + index;
      this.regions[regionName] = '#' + id;
      this.appendFormsHtml(id);
      this.addRegion(regionName, '#' + id);
      var opts = this.createFormOpts('addFetusForm', this.createNewFetusModel());
      var FormView = BaseFormView.extend(opts);
      this[regionName].show(new FormView());
      $('.title').last().focus();
    },
    appendFormsHtml: function (id) {
      var index = parseInt(id.split('-')[3]);
      var div = document.createElement('div');
      div.id = 'fetus-' + index;
      div.classList.add('header-block');
      div.classList.add('title');
      $(div).attr('tabindex', '0');
      $(div).attr('aria-live', 'assertive');
      $(div).attr('aria-label', ('fetus ' + (index + 1)));
      div.innerHTML = '<p>Fetus ' + (index + 1) + '</p>';
      this.$el.find('#forms-container').append(div);
      div = document.createElement('div');
      div.id = id;
      div.classList.add('fetuses');
      this.$el.find('#forms-container').append(div);
    },
    updateAddFetusForm: function (index) {
      for (var i = 0; i < this.forms.addFetusForm.length; i++) {
        var id = this.forms.addFetusForm[i].id.split('-');
        id[2] = index;
        this.forms.addFetusForm[i].id  = id.join('-');
      }
    },
    createFormOpts: function (id, model) {
      var form = [];
      for (var i = 0; i < this.forms[id].length; i++) {
        var obj = _.clone(this.forms[id][i]);
        form.push(obj);
      }
      return {
        template: _.template('<form class="ultrasounds-form" role="form"></form>'),
        itemViewContainer: '.ultrasounds-form',
        form: form,
        model: model
      };
    },
    createNewFetusModel: function (index) {
      var attributes = {};
      for (var i = 0; i < this.forms.addFetusForm.length; i++) {
        if (typeof index === 'undefined') {
          if (this.forms.addFetusForm[i].id.split('-')[1] === 'efw') {
            attributes[this.forms.addFetusForm[i].id] = {metric: true, weight: ''};
          } else {
            attributes[this.forms.addFetusForm[i].id] = '';
          }
        }
        else {
          attributes[this.forms.addFetusForm[i].id] =
            (typeof this.model.get('fetuses')[index][this.forms.addFetusForm[i].id.split('-')[1]] === 'undefined') ? '' :
              this.model.get('fetuses')[index][this.forms.addFetusForm[i].id.split('-')[1]];
        }
      }
      return new Backbone.Model(attributes);
    },
    showForms: function () {
      this.setAddFetusRegion();
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        var opts = '';
        if (keys[i].split('-')[0] === 'addFetusForm') {
          var index = parseInt(keys[i].split('-')[1]);
          opts = this.createFormOpts('addFetusForm', this.createNewFetusModel(index));
        } else {
          opts = this.createFormOpts(keys[i], this.model);
        }
        var FormView = BaseFormView.extend(opts);
        this[keys[i]].show(new FormView());
      }
      var regex = new RegExp('new');
      if (regex.test(Backbone.history.getFragment())) {
        $('#deleteUltrasound').parent().parent().hide();
      }
    },
    setAddFetusRegion: function () {
      for (var i = 0; i < this.model.get('fetuses').length; i++) {
        this.regions[('addFetusForm-' + i)] = ('#add-fetus-form-' + i);
        this.appendFormsHtml(('add-fetus-form-' + i));
        this.addRegion(('addFetusForm-' + i), ('#add-fetus-form-' + i));
      }
    },
    updateAddFetusModel: function (view) {
      var model = view.model.toJSON();
      var attributes = {};
      for (var i = 0; i < view.form.length; i++) {
        attributes[view.form[i].id.split('-')[1]] = model[view.form[i].id];
      }
      view.model = new Backbone.Model(attributes);
    },
    validateAddFetusForm: function (view) {
      var fetuses = view.$itemViewContainer.serializeArray();
      var efw = _.filter(fetuses, function (field) { return field.name.split('-')[1] === 'efw'; });
      var ega = _.filter(fetuses, function (field) { return field.name.split('-')[1] === 'ega'; });
      if (efw.length === 1) {
        view.form[0].value = {weight: efw[0].value, metric: false};
        view.form[1].value = {weeks: ega[0].value, days: ega[1].value};
        view.form[2].value = fetuses[3].value;
        view.form[3].value = fetuses[4].value;
      } else {
        view.form[0].value = {weight: efw[0].value, metric: (efw[1].value === 'on')};
        view.form[1].value = {weeks: ega[0].value, days: ega[1].value};
        view.form[2].value = fetuses[4].value;
        view.form[3].value = fetuses[5].value;
      }
      view.validateForm();
    },
    saveForm: function (e) {
      e.preventDefault();
      this.ultrasoundDateForm.currentView.clearErrors();
      if ($('input[name="deleteUltrasound"]').prop('checked')) {
        App.router.popups.showDeletePopup({name: 'Ultrasound', optText: 'entry'});
        this.toDelete = true;
      } else {
        this.toDelete = false;
        this.updateUltrasound();
      }
    },
    updateUltrasound: function () {
      var formName = '';
      var fetuses = [];
      this.fieldsValidated = true;
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        formName = keys[i].split('-');
        if (formName[0] === 'addFetusForm') {
          this.validateAddFetusForm(this[keys[i]].currentView);
        } else {
          this[keys[i]].currentView.updateForm();
          this[keys[i]].currentView.validateForm();
        }
        if (!this[keys[i]].currentView.fieldsValidated) {
          this.fieldsValidated = false;
        }
        if (formName[0] === 'addFetusForm' && this.fieldsValidated) {
          this.updateAddFetusModel(this[keys[i]].currentView);
          fetuses.push(this[keys[i]].currentView.model.toJSON());
        }
      }
      if (this.fieldsValidated) {
        this.model.set('fetuses', fetuses);
        this.model.set('veteranId', App.Resources.user.id);
        var self = this;
        self.model.save({}, {
          success: function () {
            if (!self.toDelete) {
              App.router.popups.showPopup({tpl: PopupTemplate});
              $('#popup-close').on('click', self.goBack);
            }
            else {
              App.vent.trigger('route:back');
            }

          }
        });
      }
    },

    goBack: function () {
      App.vent.trigger('route:back');
    },

    cancelDelete: function () {
      var $delete = $('#deleteUltrasound');
      //var label = $delete.attr('aria-label').split('.');
      //label[1] = $delete.find('.active').attr('aria-label') + ' Selected';
      //$delete.find('.aria-text').text(label[0] + ', ' + label[1] + ', ' + label[2]);
      //$delete.attr('aria-label', label.join('. '));
      $delete.focus().click();
      this.toDelete = false;
    }
  });
});

