define(function (require) {
  var App = require('app');
  var BaseFormView = require('base-form-view');
  var template = require('text!../html/vaccinationsLayoutView.html');
  var form = require('text!../json/vaccinationsForm.json');
  var PopupTemplate = require('text!../html/popup/vaccinationsToolTipView.html');

  return Backbone.Marionette.Layout.extend({
    header: {
      outer: 'Vaccinations'
    },
    template: _.template(template),

    regions: {
      fluShotForm: '#flu-shot-form',
      tdapForm: '#tdap-form'
    },

    events: {
      'click #save-btn': 'saveForm',
      'click #tdap-info-btn': 'handleTooltip',
      'keypress #tdap-info-btn': 'handleTooltip'
    },

    initialize: function () {
      this.model = App.models.vaccinations;
      this.form = JSON.parse(form);
    },
    onShow: function () {
      console.log(this.$el);
    },
    handleTooltip: function (e) {
      if (e.type === 'keypress' && e.keyCode !== 13) { return; }
      App.router.popups.showPopup({tpl: PopupTemplate});
    },
    setFluShotDefaults: function () {
      this.year = new Date().getFullYear();
      this.date = (App.Dateman.getZeroHourDate() < new Date((this.year + 1), 7)) ? new Date(this.year + 1, 8).getTime() : new Date(this.year, 8).getTime();
      this.model.set('fluShotNextDue', this.date);
      this.fluShotForm.currentView.setForm();
      this.fluShotForm.currentView.render();
    },
    setTdapDefaults: function () {
      this.lastAdmin = this.model.get('tdapLastAdmin');
      this.today = App.Dateman.getZeroHourDate().getTime();
      this.dueDate = App.Dateman.addDays(new Date(App.models.profile.get('edd')), -92).getTime(); //EGA 27 weeks
      this.deadline = App.Dateman.addDays(new Date(App.models.profile.get('edd')), -22).getTime(); //EGA 37 weeks

      if (typeof this.lastAdmin !== 'number' && this.today > this.deadline) {
        this.tdapForm.currentView.setForm();
        this.tdapForm.currentView.render();
      }
    },
    createFormOpts: function () {
      var form = [];
      for (var i = 0; i < this.form.length; i++) {
        if (this.form[i].id !== 'tdapNextDue') {
          var obj = _.clone(this.form[i]);
          form.push(obj);
        }
      }
      return {
        template: _.template('<form class="vaccinations-form" role="form"></form>'),
        itemViewContainer: '.vaccinations-form',
        form: form,
        model: this.model,
        events: {}
      };
    },
    showForms: function () {
      var keys = _.keys(this.regions);
      for (var i = 0; i < keys.length; i++) {
        if (keys[i] === 'fluShotForm') { this.updateForm('Flu Shot', 'fluShot'); }
        else { this.updateForm('Tdap', 'tdap'); }
        var opts = this.createFormOpts();
        var FormView = BaseFormView.extend(opts);
        this[keys[i]].show(new FormView());
      }
      this.setFluShotDefaults();
      this.setTdapDefaults();
    },
    updateForm: function (title, id) {
      for (var i = 0; i < this.form.length; i++) {
        if (i === 0) {
          this.form[i].id = id + 'LastAdmin';
          this.form[i].title = title + ' Last Administered';
        }
        else {
          this.form[i].id = id + 'NextDue';
          this.form[i].title = title + ' Next Due';
        }
      }
    },
    saveForm: function (e) {
      e.preventDefault();
      this.fluShotForm.currentView.clearErrors();
      this.fieldsValidated = true;
      var keys = _.keys(this.regions);
      for (var i = (keys.length - 1); i >= 0; i--) {
        this[keys[i]].currentView.updateForm();
        this[keys[i]].currentView.validateForm();
        if (!this[keys[i]].currentView.fieldsValidated) {
          this.fieldsValidated = false;
        }
      }
      this.model.set('fluShotNextDue', this.date);

      if (this.fieldsValidated) {
        this.model.set('veteranId', App.Resources.user.id);
        this.model.save({}, {
          success: function () {
            App.vent.trigger('route:back');
          }
        });
      }
    }
  });
});
